<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2014 Nagios Enterprises, LLC
//
//  File: admin_views.inc.php
//  Desc: Defines displays for all non-object configuration pages in the CCM.
//

/**
 * Displays the import config files page
 * Menu link: Import Config Files
 */ 
function ccm_import_page() 
{
    // Process input variables 
    $search = ccm_grab_request_var('txtSearch', '');
    $submitted = ccm_grab_request_var('importsubmitted', false);
    $filenames = ccm_grab_request_var('selImportFile', array());
    $overwrite = ccm_grab_request_var('chbOverwrite', false);
    $returnClass = '';

    // Feedback based on if anything was imported, import_configs() defined below
    if (!empty($filenames)) {
        $feedback = (($submitted) ? import_configs($filenames, $overwrite, $returnClass) : '');  
    }  
?>
    <div id='contentWrapper' style="width: 100%;">
        <form action='index.php' method='post' id='formInput' name='frmInput'>

            <h1 style="margin-bottom: 10px;"><?php echo gettext("Configuration Import"); ?></h1>
            <input type="text" style="width:120px" value="<?php echo $search; ?>" name="txtSearch" id='txtSearch' placeholder="Search..." />
            <img width="18" height="18" style="cursor: hand; cursor:pointer;" onclick="document.forms[0].submit()" title="Search" alt="Search" src="/nagiosql/images/lupe.gif" />
            <img width="18" height="18" style="cursor: hand; cursor:pointer;" onclick="$('#txtSearch').val('');document.forms[0].submit();" title="Delete" alt="Delete" src="/nagiosql/images/del.png">
            <br style="clear: both;" />
    
            <div id='returnContent' class='<?php echo $returnClass; ?>' <?php if (!empty($feedback)) { echo 'style="margin-bottom: 15px;"'; } ?>><?php if (!empty($feedback)) { echo $feedback; } ?></div>
                <select id="selImportFile" multiple="multiple" size="10" style='width:80%;' name="selImportFile[]">
                <?php 
                    // Fetch file list here, function defined below  
                    populate_config_files($search); 
                ?>
                </select>
                <div>
                    <label style="line-height: 30px;" for='chbOverwrite'><?php echo gettext("Overwrite Database"); ?>
                    <input type="checkbox" checked="checked" value="1" id="chbOverwrite" class="checkbox" name="chbOverwrite" style="vertical-align: middle;" /></label>
                </div>
                <p><?php echo gettext("To prevent errors or misconfigurations, you should import your configurations in an useful order. We recommend importing in the following order"); ?>:</p>
                <p>
                    <strong><em>commands -> timeperiods -> contacttemplates -> contacts -> contactgroups -> <br />
                    hosttemplates -> hosts -> hostgroups -> servicetemplates -> services -> servicegroups</em></strong>
                </p>
                <p><em><?php echo gettext("The CCM import tool does not currently support object names that start with '#', or group exclusions that start with '!'"); ?></em><br /></p>
                <p><strong><?php echo gettext("Check your configurations after import before saving to file!"); ?></strong></p>
            </div>
    
            <input type="submit" class='ccmButton' value="<?php echo gettext('Import'); ?>" id="subForm" name="subForm" />
            <input type="button" class='ccmButton' value="<?php echo gettext('Abort'); ?>" id="abort" name="abort" onclick="document.forms[0].reset();"/>
            <input type="hidden" value="true" id="importsubmitted" name="importsubmitted" />
            <input type='hidden' name='cmd' value='admin' />
            <input type='hidden' name='type' value='import' />
        
        </form>
    </div>
<?php 
}


/**
 * Display the main nagios.cfg in an editable web form.
 */
function ccm_corecfg() 
{
    global $myConfigClass;
    $myConfigClass->getConfigData("nagiosbasedir", $nagiosEtc);
    
    // Handle request variables
    $submitted = ccm_grab_request_var('submitted', false); 
    $newcfg = ccm_grab_array_var($_REQUEST, 'newcfg', '');
    $filecheck = false;
    $feedback = '';
    $contents = '';
    $returnClass = '';
    $info = gettext("This file contains all global configuration information for Nagios Core.");
    $title = gettext('Nagios Core Configuration');
    $nagioscfg = $nagiosEtc.'nagios.cfg';

    if (file_exists($nagioscfg) && is_writeable($nagioscfg)) {
        $filecheck = true;
    }

    // Save the config file
    if ($submitted && $newcfg != '') {
        if ($filecheck) {
            $r = file_put_contents($nagioscfg ,$newcfg);
            $feedback = (($r) ? gettext("File saved successfully!")."<br />" : gettext("Unabled to save file")." :$nagioscfg, ".gettext("check permissions").".");
            $returnClass = 'success';
        } else {
            $feedback= gettext("Unable to access nagios.cfg file at").": {$nagioscfg}.  ".gettext("Check permissions and verify write access").".";
            $returnClass = 'error';
        }
        
        $feedback .= "<div id='closeReturn'>
                        <a href='javascript:void(0)' id='closeReturnLink' title='Close'>".gettext("Close")."</a>
                    </div>";
    }

    if ($filecheck) {
        $contents = file_get_contents($nagioscfg);
    } else {
        $contents = gettext("ERROR: Unable to read / write nagios.cfg file. Check permissions!");
    }

    // Begin page content      
    text_editor_page($title, $returnClass, $feedback, $contents, $info, 'corecfg');
}


/**
 * Display the cgi.cfg in an editable web form.
 */
function ccm_cgicfg() 
{
    global $myConfigClass;
    $myConfigClass->getConfigData("nagiosbasedir", $nagiosEtc);

    // Handle request variables
    $submitted = ccm_grab_request_var('submitted', false);
    $newcfg = ccm_grab_array_var($_REQUEST, 'newcfg', '');
    $filecheck = false;
    $feedback = '';
    $contents = '';
    $returnClass = '';
    $info = gettext("This file contains all CGI configuration information for Nagios Core.");
    $title = gettext('Nagios Core CGI Config');
    $nagioscfg = $nagiosEtc.'cgi.cfg';

    if (file_exists($nagioscfg) && is_writeable($nagioscfg)) {
        $filecheck = true;
    }

    // Save the config file
    if ($submitted && $newcfg != '') {
        if ($filecheck) {
            $r = file_put_contents($nagioscfg,$newcfg); 
            $feedback = ($r) ? gettext("File saved successfully!")."<br />" : gettext("Unable to save file")." :$nagioscfg, ".gettext("check permissions")."."; 
            $returnClass = 'success';
        }               
        else {
            $feedback= gettext("Unable to access nagios.cfg file at").": {$nagioscfg}.  ".gettext("Check permissions and verify write access").".";
            $returnClass = 'error'; 
        }   
        
        $feedback .= "<div id='closeReturn'>
                        <a href='javascript:void(0)' id='closeReturnLink' title='".gettext("Close")."'>".gettext("Close")."</a>
                    </div>";
    }

    if ($filecheck) {
        $contents = file_get_contents($nagioscfg);
    } else {
        $contents = gettext("ERROR: Unable to read / write cgi.cfg file.  Check permissions!");
    }

    // Begin page content      
    text_editor_page($title, $returnClass, $feedback, $contents, $info, 'cgicfg');
}


/**
 * Displays a textarea with a config file to be edited.
 *
 * @param string $title Title of the config that's being edited
 * @param string $returnClass The name of the style class of the return value (i.e. error or message)
 * @param string $feedback The message passed from saving the config or performing an action
 * @param string $info An extra string to display any HTML text
 * @param string $type The type of config being edited ('corecfg' or 'cgicfg')
 */
function text_editor_page($title, $returnClass, $feedback, $contents, $info, $type) 
{
?>
    <div id='contentWrapper'>
        <h1 style="margin-bottom: 10px;"><?php echo $title; ?></h1>
        <div id='returnContent' class='<?php echo $returnClass; ?>' style="margin-bottom: 10px;">
            <?php echo $feedback; ?>
        </div>
        <p><?php echo $info; ?></p>
        <div id='centerdiv'>
            <form action='index.php' method='post' id='formInput' name='frmInput'>
                <div>
                    <textarea name="newcfg" style="width: 600px; height: 600px;" class="monospace-textarea" id='newcfg'><?php echo $contents; ?></textarea>
                </div>
                <div style="margin-top: 15px;">
                    <input type="submit" class='ccmButton' value="<?php echo gettext("Save"); ?>" id="subForm" name="subForm" />
                    <input type="button" class='ccmButton' value="<?php echo gettext("Abort"); ?>" id="reset" name="reset" onclick="javascript:window.location='index.php';"/>
                    <input type="hidden" value="true" id="submitted" name="submitted" />
                    <input type='hidden' name='cmd' value='admin' />
                    <input type='hidden' name='type' value='<?php echo $type; ?>' />
                </div> 
            </form>
        </div>
    </div>
<?php
}


/**
 * Creates html for add/edit user form. Is not available for people who have Nagios XI
 * integration installed because they will never see the users page.
 */ 
function manage_user_html()
{
    global $ccmDB;
    global $CFG;
    $mode = ccm_grab_request_var('cmd', 'insert');
    $id = ccm_grab_request_var('id', false);

    // Defaults
    $username = '';
    $alias = ''; 
    $password = ccm_random_string(8);
    $confirm = ''; 
    $active = 'checked="checked"';
    $languages = $CFG['languages'];
    $lang = 'en_US';

    // Not used in CCM but exists for old NagiosQL
    $access = '11111111';
    $ws_auth = false;

    // Editing an existing user 
    if ($mode == 'modify' && $id != '') {
        $query = "SELECT * FROM tbl_user WHERE id='$id';";
        $array = $ccmDB->query($query);

        $username = ccm_grab_array_var($array[0], 'username', '');
        $alias = ccm_grab_array_var($array[0], 'alias', ''); 
        $active = (($array[0]['active'] == 1) ? 'checked="checked"' : '');
        $lang = ccm_grab_array_var($array[0], 'locale', 'en_US');
    }
?>  
    <div id='tab1'>
        <div style="margin-bottom: 10px;">
            <label for='username'>*<?php echo gettext("Username"); ?>:</label>
            <div><input type='text' class='required' name='username' id='username' value='<?php echo $username; ?>'></div>
        </div>
        <div style="margin-bottom: 10px;">
            <label for='alias'><?php echo gettext("Alias"); ?>: </label>
            <div><input type='text' name='alias' id='alias' value='<?php echo $alias; ?>'></div>
        </div>
        <div style="margin-bottom: 10px;">
            <label for='password'>*<?php echo gettext("Password"); ?>:</label>
            <div><input type='password' name='password' class='required' id='password' value='<?php echo $password; ?>'></div>
            <label for='config'>*<?php echo gettext("Confirm Password"); ?>:</label>
            <div><input type='password' name='confirm' class='required' id='confirm' value='<?php echo $confirm; ?>'></div>
        </div>
        <div style="margin-bottom: 10px;">
            <input type='checkbox' name='active' id='active' value='1' <?php echo $active; ?>>
            <label for='active'> <?php echo gettext("Active"); ?> </label>
        </div>
        <div>
            <label for='lang'><?php echo gettext("Language"); ?></label>
            <div>
                <select name='lang' id='lang' style="width:100px;">
                    <?php
                    foreach ($languages as $l) {
                        echo "<option value='{$l}'";
                        if ($lang == $l) { echo " selected='selected' "; }
                        echo ">{$l}</option>";
                    }   
                    ?>
                </select>
            </div>
            <input type='hidden' name='id' value='<?php echo $id; ?>'>
        </div>
    </div>
<?php
}

/**
 * Handle add/insert submission for a CCM user
 * @return mixed array(int $errors, string $message) 
 */
function process_user_submission()
{
    global $myDataClass;

    // Return variables 
    $errors = 0;
    $message ='';

    // Process input variables 
    $mode = ccm_grab_request_var('mode', 'insert');
    $id = ccm_grab_request_var('id', false);
    $cmd = ccm_grab_request_var('cmd', false);

    // Form defaults 
    $username = ccm_grab_request_var('username', '');
    $alias = ccm_grab_request_var('alias', ''); 
    $password=ccm_grab_request_var('password', '');
    $active = ccm_grab_request_var('active', false);
    $lang = ccm_grab_request_var('lang', 'en_US');

    // Not used in CCM but exists for old NagiosQL
    $access = '11111111';
    $ws_auth = false;

    // Statements below modified from original admin/user.php    
    $strSQLx = "`tbl_user` SET `username`='$username', `alias`='$alias', `access_rights`='$access', `password`=MD5('$password'), `wsauth`='$ws_auth', `active`='$active', `locale`='{$lang}', `last_modified`=NOW()";
    if ($mode == "insert" && $cmd != 'delete') {
        $strSQL = "INSERT INTO ".$strSQLx;
    } else if ($cmd == 'delete') {
        $strSQL = "DELETE FROM tbl_user WHERE `id`='{$id}'";
    } else {
        $strSQL = "UPDATE ".$strSQLx." WHERE `id`=$id";
    }

    if ($mode == "modify" && $username == $_SESSION['username']) {
        ccm_set_language($lang);
    }

    // Error handling 
    $errors = $myDataClass->dataInsert($strSQL, $intInsertId);
    $message = (($errors > 0)? $myDataClass->strDBMessage : gettext('User updated successfully!'));
    
    // Logging
    if ($mode == "insert") { 
        $myDataClass->writeLog(gettext('A new user added').": ".$username);
    } else if ($mode == "modify") {
        $myDataClass->writeLog(gettext('User modified').": ".$username);
    }

    return array($errors, $message);
}


/**
 * Updates global CCM settings based on request vars
 */
function update_ccm_settings() 
{
    global $CFG;
    global $myDBClass;
    global $ccmDB;

    $errors = 0;
    $msg = '';

    $txtRootPath = ccm_grab_request_var('txtRootPath');
    $txtBasePath = ccm_grab_request_var('txtBasePath');
    $txtTempDir = ccm_grab_request_var('txtTempDir');
    $selProtocol = ccm_grab_request_var('selProtocol');
    $txtDBserver = ccm_grab_request_var('txtDBserver');
    $txtDBport = ccm_grab_request_var('txtDBport');
    $txtDBname = ccm_grab_request_var('txtDBname');
    $txtDBuser = ccm_grab_request_var('txtDBuser');
    $txtDBpassword = ccm_grab_request_var('txtDBpassword');
    $txtLogoff = ccm_grab_request_var('txtLogoff');
    $txtLines = ccm_grab_request_var('txtLines');
    $txtStaticDir = ccm_grab_request_var('txtStaticDir');
    
    // Write global settings to database
    $strSQL = "SET @previous_value := NULL";
    $booReturn = $myDBClass->insertData($strSQL);
    $strSQL  = "INSERT INTO `tbl_settings` (`category`,`name`,`value`) VALUES";
    $strSQL .= "('path','root','".str_replace("\\", "\\\\", $txtRootPath)."'),";
    $strSQL .= "('path','physical','".str_replace("\\", "\\\\", $txtBasePath)."'),";
    $strSQL .= "('path','protocol','".$selProtocol."'),";
    $strSQL .= "('path','tempdir','".str_replace("\\", "\\\\", $txtTempDir)."'),";
    
    // Added option for static config directory 
    $strSQL .= "('path','staticdir','".str_replace("\\", "\\\\", $txtStaticDir)."'),";
    $strSQL .= "('security','logofftime','".$txtLogoff."'),";
    $strSQL .= "('common','pagelines','".$txtLines."')";
    $strSQL .= "ON DUPLICATE KEY UPDATE value = IF((@previous_value := value) <> NULL IS NULL, VALUES(value), NULL);";
    $booReturn = $myDBClass->insertData($strSQL);
    if ($booReturn == false) {
        $errors++;
        $msg = gettext("An error occured while writing settings to database")."<br>".$myDBClass->strDBError;
    }
    $strSQL = "SELECT @previous_note";
    $booReturn = $myDBClass->insertData($strSQL);
    
    // Update the settings.php file 
    $file = $txtBasePath."config/settings.php";
    if (file_exists($file) && is_writeable($file)) {
        $string = "
<?php
exit;
?>

[db]
server   = ".$txtDBserver."
port     = ".$txtDBport."  
database = ".$txtDBname."
username = ".$txtDBuser."
password = ".$txtDBpassword."

[common]
install  = passed";
        file_put_contents($file, $string);
    } else {
        $errors++;
        $msg .= gettext("Unable to save to file").": $file<br/>";
    }
        
    // Rebuild $CFG array after settings update
    $CFG['settings'] = array();
    $settings = $ccmDB->query("SELECT * FROM tbl_settings;");
    foreach ($settings as $s) {
        $CFG[$s['category']][$s['name']] = $s['value'];
    }

    // Update session settings 
    $_SESSION['SETS'] = $CFG;

    if ($errors == 0) {
        $msg = gettext('Settings updated successfully!');
    }

    return array($errors, $msg);
}


/**
 * Displays global CCM settings page form.
 * Menu link: Config Manager Settings
 */ 
function ccm_settings()
{
    global $CFG;
    $errors = 0;
    $msg = '';
    $submitted = ccm_grab_request_var('submitted', false);
    $returnClass = 'hidden';

    if ($submitted) {
        list($errors, $msg) = update_ccm_settings(); 
        $returnClass = (($errors > 0) ? 'error' : 'success'); 
    }
    $https = (($CFG['path']['protocol'] == 'https') ? "selected='selected'" : '');
    $http = (($CFG['path']['protocol'] == 'http') ? "selected='selected'" : ''); 
    $staticDir = (isset($CFG['path']['staticdir']) ? $CFG['path']['staticdir'] : '/usr/local/nagios/etc/static');
?>
    <div id='contentWrapper'>
        <h2><?php echo gettext("Global CCM Settings"); ?></h2>
        <div id='returnContent' class='<?php echo $returnClass; ?>'>
            <?php echo $msg; ?>
        </div>
        <form action='index.php' method='post' id='formInput' name='frmInput'>
            <h4><?php echo gettext("Paths"); ?>:</h4>
            <div>
                <div class="content-row">
                    <label for='txtRootPath'><?php echo gettext("Application Root Path"); ?>*</label>
                    <div><input type="text" style="width:300px" class="required" value="<?php echo $CFG['path']['root']; ?>" id="txtRootPath" name="txtRootPath"></div>
                </div>
                <div class="content-row">
                    <label for='txtBasePath'><?php echo gettext("Application Base Path"); ?>*</label>
                    <div><input type="text" style="width:300px" class="required" value="<?php echo $CFG['path']['physical']; ?>" id="txtBasePath" name="txtBasePath"></div>
                </div>
                <div class="content-row">
                    <label for='txtTempDir'><?php echo gettext("Temp Directory"); ?>*</label>
                    <div><input type="text" style="width:300px" class="required" value="<?php echo $CFG['path']['tempdir']; ?>" id="txtTempDir" name="txtTempDir"></div>
                </div>
                <div class="content-row">
                    <label for='txtStaticDir'><?php echo gettext("Static Configuration Directory"); ?>*</label>
                    <div><input type="text" style="width:300px" class="required" value="<?php  echo $staticDir; ?>" id="txtStaticDir" name="txtStaticDir"></div>
                </div>
                <div>
                    <label for='selProtocol'><?php echo gettext("Server Protocol"); ?>*</label>
                    <div>
                        <select id="selProtocol" name="selProtocol">
                            <option value='http' <?php echo $http; ?> >HTTP</option>
                            <option value='https' <?php echo $https; ?> >HTTPS</option>
                        </select>
                    </div>
                </div>
            </div>
            <h4><?php echo gettext("Database"); ?>:</h4>
            <div>
                <div class="content-row">
                    <label for='txtDBserver'><?php echo gettext("MySQL Server"); ?>*</label>
                    <div><input type="text" style="width:200px" class="required" value="<?php echo $CFG['db']['server']; ?>" id="txtDBserver" name="txtDBserver"></div>
                </div>
                <div class="content-row">
                    <label for='txtDBport'><?php echo gettext("MySQL Server Port"); ?>*</label>
                    <div><input type="text" style="width:200px" class="required" value="<?php echo $CFG['db']['port']; ?>" id="txtDBport" name="txtDBport"></div>
                </div>   
                <div class="content-row">
                    <label for='txtDBname'><?php echo gettext("Database Name"); ?>*</label>
                    <div><input type="text" style="width:200px" class="required" value="<?php echo $CFG['db']['database']; ?>" id="txtDBname" name="txtDBname"></div>
                </div>
                <div class="content-row">
                    <label for='txtDBuser'><?php echo gettext("Database User"); ?>*</label>
                    <div><input type="text" style="width:200px" class="required" value="<?php echo $CFG['db']['username']; ?>" id="txtDBuser" name="txtDBuser"></div>
                </div>
                <div>
                    <label for='txtDBpassword'><?php echo gettext("Database Password"); ?>*</label>
                    <div><input type="password" style="width:200px" class="required" value="<?php echo $CFG['db']['password']; ?>" id="txtDBpassword" name="txtDBpassword"></div>
                </div>
            </div>
            <h4><?php echo gettext("Common"); ?></h4>
            <div>
                <?php
                $show_session_limit = true;
                if (ENVIRONMENT == "nagiosxi") {
                    $separate_ccm_login = get_option("separate_ccm_login", 0);
                    if (!$separate_ccm_login) {
                        $show_session_limit = false;
                    }
                }

                if ($show_session_limit) {
                ?>
                <div class="content-row">
                    <label for='txtLogoff'><?php echo gettext("Session Auto Logout Time"); ?></label>
                    <div><input type="text" style="width:200px" value="<?php echo $CFG['security']['logofftime']; ?>" id="txtLogoff" name="txtLogoff"></div>
                </div>
                <?php } ?>
                <div>
                    <label for='txtLines'><?php echo gettext("Default Result Limit"); ?></label>
                    <div><input type="text" style="width:200px" value="<?php echo  $CFG['common']['pagelines']; ?>" id="txtLines" name="txtLines"></div>
                </div>
            </div>
            <div style="margin-top: 15px;">
                <input type="submit" class='ccmButton' value="<?php echo gettext("Save"); ?>" id="subForm" name="subForm" />
                <input type="button" class='ccmButton' value="<?php echo gettext("Abort"); ?>" id="abort" name="abort" onclick="window.location='index.php';"/>
                <input type="hidden" value="true" id="submitted" name="submitted" />
                <input type='hidden' name='cmd' value='admin' />
                <input type='hidden' name='type' value='settings' />
            </div>
        </form>
    </div>
<?php 
}


/**
 * Displays the page that allows editing of static files from the static files directory.
 * Menu link: Static Configurations
 */
function ccm_static_editor()
{
    global $ccmDB;
    $returnClass = 'hidden';
    $feedback = '';
    $submitted = ccm_grab_request_var('submitted', false);

    // Get config option for static directory 
    $query = "SELECT `value` FROM tbl_settings WHERE `name`='staticdir'";
    $dir = $ccmDB->query($query);
    $staticDir = (isset($dir[0]['value']) ? $dir[0]['value'] : '/usr/local/nagios/etc/static');

    // Get static files, dump to $output variable
    ccm_dir_to_array($staticDir, '', '', $output, $errMessage);
    
    // Save static file
    if ($submitted) {
        $newcfg = ccm_grab_request_var('newcfg', '');
        $file = urldecode(ccm_grab_request_var('staticFile', ''));

        if (is_writeable($file)) {
            $newcfg = str_replace('\r\n', "\n", $newcfg);
            if (!file_put_contents($file, $newcfg)) {
                $feedback = gettext("Unable to write to file").": $file. ".gettext("Check permissions");
                $returnClass = 'error';
            } else {
                $feedback = gettext("File").": <strong>$file</strong> ".gettext("saved successfully!");
                $returnClass = 'success';
            }
        }
    }
?>
    <div id='contentWrapper'>
        <h1><?php echo gettext("Static Configuration"); ?></h1>
        <div id='returnContent' class='<?php echo $returnClass; ?>'>
            <?php echo $feedback; ?>
            <div id='closeReturn'>
                <a href='javascript:void(0)' id='closeReturnLink' title='Close'><?php echo gettext("Close"); ?></a>
            </div>
        </div>
        <p><?php echo gettext("This tool allows editing of configuration files that are NOT stored in the Nagios CCM database"); ?>.</p>
        <div id='centerdiv'>
            <form action='index.php' method='post' id='formInput' name='frmInput'>
                <div class="content-row">
                    <label for='staticFiles'><?php echo gettext("Static Files"); ?>: <?php echo $staticDir; ?></label>
                    <div>
                        <select id='staticFiles' name='staticFile'>
                        <?php
                        foreach ($output as $file) {
                            echo "<option value='".urlencode($file)."'>$file</option>";
                        }
                        ?>  
                        </select>
                        <input type='button' id='loadValue' value='<?php echo gettext('Load File'); ?>' class='ccmButton' onclick='getStaticFile()' />
                    </div>
                </div>
                <div>
                    <textarea name="newcfg" style="width: 600px; height: 600px;" id='newcfg' class="monospace-textarea"></textarea>
                </div>
                <div style="margin-top: 15px;">
                    <input type="submit" class='ccmButton' value="<?php echo gettext("Save"); ?>" id="subForm" name="subForm" />
                    <input type="button" class='ccmButton' value="<?php echo gettext("Abort"); ?>" id="reset" name="reset" onclick="javascript:window.location='index.php';"/>
                    <input type="hidden" value="true" id="submitted" name="submitted" />
                    <input type='hidden' name='cmd' value='admin' />
                    <input type='hidden' name='type' value='static' />
                </div>
            </form>
        </div>
    </div>
<?php
}


/**
 * Imports nagios configs based on request variable array of files modified from original import.php script
 *
 * @param mixed $files Array of files to import
 * @param int $chkOverwrite Checkbox option to overwrite existing DB info
 * @param string $returnClass REFERENCE variable for div's CSS class ... "success" or "error"
 * @return string $message Feedback message to tell if imports were all successful
 */
function import_configs($files, $chkOverwrite, &$returnClass)
{
    global $myDataClass;
    global $myImportClass;
    $imported_files = 0;
    $message = '';
    $errors = 0;

    // Process selected files for import 
    if (!empty($files)) {       
        foreach ($files AS $elem) {
            $intReturn = $myImportClass->fileImport($elem, $chkOverwrite);
            $myDataClass->writeLog(gettext('File imported - File [overwite flag]:')." ".$elem." [".$chkOverwrite."]");
            if ($intReturn == 1) {
                $message .= $myImportClass->strDBMessage . $myImportClass->strMessage . "<br>";
                $errors++;
            } else {
                $imported_files++;
            }
        }
    }

    if ($errors == 0) {
        $returnClass = 'success';
        $message = $imported_files. " ".gettext("file(s) imported successfully!")."<br>";
    } else {
        $returnClass = 'error';
        $message .= $errors." ".gettext("items failed to import successfully")."<br>";
    }
    return $message;
}


/**
 * Writes the changes made in the textarea page to the files.
 *
 * @param string $chkSearch
 */
function populate_config_files($chkSearch) 
{
    global $myConfigClass;
    $myConfigClass->getConfigData("basedir", $strBaseDir);
    $myConfigClass->getConfigData("hostconfig", $strHostDir);
    $myConfigClass->getConfigData("serviceconfig", $strServiceDir);
    $myConfigClass->getConfigData("backupdir", $strBackupDir);
    $myConfigClass->getConfigData("hostbackup", $strHostBackupDir);
    $myConfigClass->getConfigData("servicebackup", $strServiceBackupDir);
    $myConfigClass->getConfigData("importdir", $strImportDir);
    $myConfigClass->getConfigData("nagiosbasedir", $strNagiosBaseDir);

    // Building local file list
    $output = array();
    $temp = ccm_dir_to_array($strBaseDir, "\.cfg", "cgi.cfg|nagios.cfg|nrpe.cfg|nsca.cfg|ndo2db.cfg|ndomod.cfg|resource.cfg", $output, $errMessage);
    if ($strNagiosBaseDir != $strBaseDir) {
        $temp = ccm_dir_to_array($strNagiosBaseDir, "\.cfg", "cgi.cfg|nagios.cfg|nrpe.cfg|nsca.cfg|ndo2db.cfg|ndomod.cfg|resource.cfg", $output, $errMessage);
    }
    
    $temp = ccm_dir_to_array($strHostDir, "\.cfg", "", $output, $errMessage);
    $temp = ccm_dir_to_array($strServiceDir, "\.cfg", "", $output, $errMessage);
    $temp = ccm_dir_to_array($strHostBackupDir, "\.cfg_", "", $output, $errMessage);
    $temp = ccm_dir_to_array($strServiceBackupDir, "\.cfg_", "", $output, $errMessage);
    
    if (($strImportDir != "") && ($strImportDir != $strBaseDir) && ($strImportDir != $strNagiosBaseDir)) {
        $temp = ccm_dir_to_array($strImportDir, "\.cfg", "", $output, $errMessage);
    }
    
    $output = array_unique($output);
    if (is_array($output) && (count($output) != 0)) {
        foreach ($output AS $elem) {
            if (($chkSearch == "") || (substr_count($elem,$chkSearch) != 0))  {
                echo "<option value='$elem'>$elem</option>";       
            }
        }
    }
}


/**
 * Function to add files of a given directory to an array
 *
 * @param string $sPath
 * @param string $include string match to include
 * @param string $exclude expression match to exclude
 * @param string $output  REFERENCE variable to output
 * @param        $errMessage
 *
 * @internal param string $errMEssage REFERENCE variable to error output message
 */
function ccm_dir_to_array($dir, $include, $exclude, &$output, &$errMessage) 
{
    while (substr($dir, -1) == "/" || substr($dir, -1) == "\\") {
        $dir = substr($dir, 0, -1);
    }

    $handle = @opendir($dir);
    if ($handle === false) {
        $errMessage = gettext('Could not open directory').": ".$dir;
    } else {
        while ($arrDir[] = readdir($handle)) { }
        closedir($handle);
        sort($arrDir);
        foreach($arrDir as $file) {
            if (!preg_match("/^\.{1,2}/", $file) and strlen($file)) {
                if (is_dir($dir."/".$file) && strpos($file, 'static') === false && strpos($file, 'pnp') === false) {
                    ccm_dir_to_array($dir."/".$file, $include, $exclude, $output, $errMessage);
                } else {
                    if (preg_match("/".$include."/", $file) && (($exclude == "") || !preg_match("/".$exclude."/", $file))) {
                        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
                            $dir = str_replace("/", "\\", $dir);
                            $output [] = $dir."\\".$file;
                        } else {
                            $output [] = $dir."/".$file;
                        }
                    }
                }
            }
        }
    }
}