<?php
// AVAILABILITY REPORT
//
// Copyright (c) 2010 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: availability.php 3088 2015-01-07 17:00:16Z jomann $

ini_set('display_errors', 'off'); //graphs will not generate if error messaging turned on

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// initialization stuff
pre_init();

// start session
init_session();

// grab GET or POST variables 
grab_request_vars();

// check prereqs
check_prereqs();

// check authentication
check_authentication(false);


route_request();

function route_request()
{

    $host = grab_request_var("host", "");
    $service = grab_request_var("service", "");
    $hostgroup = grab_request_var("hostgroup", "");
    $servicegroup = grab_request_var("servicegroup", "");

    // check perms
    $auth = true;
    if ($service != "")
        $auth = is_authorized_for_service(0, $host, $service);
    else if ($host != "")
        $auth = is_authorized_for_host(0, $host);
    else if ($hostgroup != "")
        $auth = is_authorized_for_hostgroup(0, $hostgroup);
    else if ($servicegroup != "")
        $auth = is_authorized_for_servicegroup(0, $servicegroup);
    if ($auth == false) {
        echo gettext("ERROR: You are not authorized to view this report.");
        exit;
    }

    $mode = grab_request_var("mode", "");
    switch ($mode) {
        case "csv":
            get_availability_csv();
            break;
        case "pdf":
            get_availability_pdf();
            break;
        case "getchart":
            get_chart_image();
            break;
        default:
            display_availability();
            break;
    }
}


///////////////////////////////////////////////////////////////////
// CHART FUNCTIONS
///////////////////////////////////////////////////////////////////


function get_chart_image()
{

    $width = grab_request_var("width", 400);
    $height = grab_request_var("height", 300);
    $title = grab_request_var("title", "");
    $rawdata = grab_request_var("data", "");

    /*
    $rawcolors=grab_request_var("colors","");
    if($rawcolors=="")
        $colors=array('1E90FF','2E8B57','ADFF2F','DC143C','BA55D3');
    else
        $colors=explode(",",$rawcolors);
    $rawlegend=grab_request_var("legend","");
    if($rawlegend=="")
        $legend=array();
    else
        $legend=explode(",",$rawlegend);
    */
    // Get a Highcharts graph instead

    // If no Raw Data
    if ($rawdata == "") {
        $data = array();
    } else {
        $rawdata = explode(",", $rawdata);
        $data = $rawdata;
    }

    $json = array('data' => $data,
        'graph_title' => $title,
        'width' => $width,
        'height' => $height);

    print json_encode($json);
    die();
}

///////////////////////////////////////////////////////////////////
// BACKEND DATA FUNCTIONS
///////////////////////////////////////////////////////////////////

// this function gets state history data in XML format from the backend
/**
 * @param string $type
 * @param        $args
 * @param        $data
 *
 * @return bool
 */
function get_availability_data($type = "host", $args, &$data)
{

    $data = get_xml_availability($type, $args);

    return true;
}

///////////////////////////////////////////////////////////////////
// REPORT GENERATION FUCNTIONS
///////////////////////////////////////////////////////////////////

// this function displays event log data in HTML
function display_availability()
{
    global $request;

    // makes sure user has appropriate license level
    licensed_feature_check();

    // get values passed in GET/POST request
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");

    // fix custom dates
    if ($reportperiod == "custom") {
        if ($enddate == "") {
            $enddate = strftime("%c", time());
        }
        if ($startdate == "") {
            $startdate = strftime("%c", time() - (60 * 60 * 24));
            $enddate = strftime("%c", time());
        }
    }

    $host = grab_request_var("host", "");
    $service = grab_request_var("service", "");
    $hostgroup = grab_request_var("hostgroup", "");
    $servicegroup = grab_request_var("servicegroup", "");

    // Grab hideoptions
    $hideoptions = grab_request_var("hideoptions", 0);
    $showonlygraphs = grab_request_var("showonlygraphs", 0);

    // should we show detail by default?
    $showdetail = 1;
    if ($host == "" && $service == "" && $hostgroup == "" && $servicegroup == "")
        $showdetail = 0;

    $showdetail = grab_request_var("showdetail", $showdetail);

    // determine start/end times based on period
    get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

    // ADVANCED OPTIONS
    $timeperiod = grab_request_var("timeperiod", "");
    $assumeinitialstates = grab_request_var("assumeinitialstates", "yes");
    $assumestateretention = grab_request_var("assumestateretention", "yes");
    $assumestatesduringdowntime = grab_request_var("assumestatesduringdowntime", "yes");
    $includesoftstates = grab_request_var("includesoftstates", "no");
    $assumedhoststate = grab_request_var("assumedhoststate", 3);
    $assumedservicestate = grab_request_var("assumedservicestate", 6);
    $advanced = grab_request_var("advanced", 0);
    $manual_run = grab_request_var("manual_run", 0);
    $display_service_graphs = grab_request_var("servicegraphs", 0);
    $dont_count_downtime = grab_request_var("dont_count_downtime", 0);

    // determine title
    if ($service != "")
        $title = gettext("Service Availability");
    else if ($host != "")
        $title = gettext("Host Availability");
    else if ($hostgroup != "")
        $title = gettext("Hostgroup Availability");
    else if ($servicegroup != "")
        $title = gettext("Servicegroup Availability");
    else
        $title = gettext("Availability Summary");


    // start the HTML page
    if (!$showonlygraphs) {
        do_page_start(array("page_title" => $title), true);
    }

    /* TESTING */
    /*
    echo "REPORTPERIOD: $reportperiod<BR>";
    echo "STARTDATE: $startdate<BR>";
    echo "ENDDATE: $enddate<BR>";
    echo "STARTTIME: $starttime<BR>";
    echo "ENDTIME: $endtime<BR>";
    echo "START: ".strftime("%c",$starttime)."<BR>";
    echo "END: ".strftime("%c",$endtime)."<BR>";
    */

    ?>

<?php

// LOGO FOR GENERATED PDFS
    if ($hideoptions && !$showonlygraphs) {

        // default logo stuff
        $logo = "nagiosxi-logo-small.png";
        $logo_alt = get_product_name();

        // use custom logo if it exists
        $logosettings_raw = get_option("custom_logo_options");
        if ($logosettings_raw == "") {
            $logosettings = array();
        } else {
            $logosettings = unserialize($logosettings_raw);
        }

        $custom_logo_enabled = grab_array_var($logosettings, "enabled");
        if ($custom_logo_enabled == 1) {
            $logo = grab_array_var($logosettings, "logo", $logo);
            $logo_alt = grab_array_var($logosettings, "logo_alt", $logo_alt);
        }

        ?>

        <div style="padding-bottom: 10px;">
            <div style="float: left; margin-right: 30px;">
                <img src="<?php echo get_base_url(); ?>images/<?php echo $logo; ?>" border="0"
                     alt="<?php echo $logo_alt; ?>" title="<?php echo $logo_alt; ?>">
            </div>
            <div style="float: left; height: 44px;">
                <div style="font-weight: bold; font-size: 22px; padding-bottom: 4px;"><?php echo $title; ?></div>
                <div><?php echo gettext("From"); ?>:
                    <strong><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong> <?php echo gettext("To"); ?>
                    : <strong><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong>
                </div>
            </div>
            <div style="clear:both;"></div>
        </div>

    <?php
    } else if (!$showonlygraphs) { // If not hideoptions, display normally
        ?>

        <h1><?php echo $title; ?></h1>

    <?php
    } // End hideoptions (display pdf report)


    if (!$showonlygraphs) {
        if ($service != "") {
            ?>
            <div class="servicestatusdetailheader" style="padding: 0; margin: 20px 0 0 0;">
                <div class="serviceimage">
                    <!--image-->
                    <?php show_object_icon($host, $service, true); ?>
                </div>
                <div class="servicetitle">
                    <div class="servicename"><a
                            href="<?php echo get_service_status_detail_link($host, $service); ?>"><?php echo htmlentities($service); ?></a>
                    </div>
                    <div class="hostname"><a
                            href="<?php echo get_host_status_detail_link($host); ?>"><?php echo htmlentities($host); ?></a>
                    </div>
                </div>
            </div>
            <br clear="all">

        <?php
        } else if ($host != "") {
            ?>
            <div class="hoststatusdetailheader" style="padding: 0; margin: 20px 0 0 0;">
                <div class="hostimage">
                    <!--image-->
                    <?php show_object_icon($host, "", true); ?>
                </div>
                <div class="hosttitle">
                    <div class="hostname"><a
                            href="<?php echo get_host_status_detail_link($host); ?>"><?php echo htmlentities($host); ?></a>
                    </div>
                </div>
            </div>
            <br clear="all">
        <?php
        } else if ($hostgroup != "") {
            ?>
            <div class="hoststatusdetailheader" style="padding: 0; margin: 20px 0 0 0;">
                <div class="hosttitle">
                    <div class="hostname"><?php echo htmlentities($hostgroup); ?></div>
                </div>
            </div>
        <?php
        } else if ($servicegroup != "") {
            ?>
            <div class="hoststatusdetailheader" style="padding: 0; margin: 20px 0 0 0;">
                <div class="hosttitle">
                    <div class="hostname"><?php echo htmlentities($servicegroup); ?></div>
                </div>
            </div>
        <?php
        }
    }
    ?>

<?php
// Hide options for rendering reports
    if (!$hideoptions) {
        if ($service != "") {
            $url = "statehistory.php?host=" . urlencode($host) . "&service=" . urlencode($service) . "&reportperiod=" . urlencode($reportperiod) . "&startdate=" . urlencode($startdate) . "&enddate=" . urlencode($enddate);
            echo "<a href='" . $url . "'><img src='" . theme_image("history.png") . "' alt='" . gettext("View State History") . "' title='" . gettext("View State History") . "'></a>";
        } else if ($host != "") {
            $url = "statehistory.php?host=" . urlencode($host) . "&reportperiod=" . urlencode($reportperiod) . "&startdate=" . urlencode($startdate) . "&enddate=" . urlencode($enddate);
            echo "<a href='" . $url . "'><img src='" . theme_image("history.png") . "' alt='" . gettext("View State History") . "' title='" . gettext("View State History") . "'></a>";
        }

        ?>
        <form method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]); ?>">
        <input type="hidden" name="host" value="<?php echo htmlentities($host); ?>">
        <input type="hidden" name="service" value="<?php echo htmlentities($service); ?>">

        <div class="reportexportlinks">
            <?php echo get_add_myreport_html($title, $_SERVER["REQUEST_URI"], array()); ?>
            <?php
            $url = "?1";
            foreach ($request as $var => $val)
                $url .= "&" . urlencode($var) . "=" . urlencode($val);
            ?>
            <?php
            if ($service != ""){
            ?>
            <a href="<?php echo $url; ?>&mode=csv" alt="<?php echo gettext("Export As CSV"); ?>"
               title="<?php echo gettext("Export As CSV"); ?>"><img src="<?php echo theme_image("csv.png"); ?>">
                <?php
                }
                else{
                ?>
                <a href="<?php echo $url; ?>&mode=csv&csvtype=host"
                   alt="<?php echo gettext("Export Host Data As CSV"); ?>"
                   title="<?php echo gettext("Export Host Data As CSV"); ?>"><img
                        src="<?php echo theme_image("csv.png"); ?>">
                    <a href="<?php echo $url; ?>&mode=csv&csvtype=service"
                       alt="<?php echo gettext("Export Service Data As CSV"); ?>"
                       title="<?php echo gettext("Export Service Data As CSV"); ?>"><img
                            src="<?php echo theme_image("csv.png"); ?>">
                        <?php
                        }
                        ?>
                        <a href="<?php echo $url; ?>&mode=pdf" alt="<?php echo gettext("Download As PDF"); ?>"
                           title="<?php echo gettext("Download As PDF"); ?>"><img
                                src="<?php echo theme_image("pdf.png"); ?>"></a>
        </div>

        <div class="reportsearchbox">
        </div>

        <?php
        $auto_start_date = get_datetime_string(strtotime('yesterday'), DT_SHORT_DATE);
        $auto_end_date = get_datetime_string(strtotime('Today'), DT_SHORT_DATE);
        ?>

        <script type="text/javascript">
            $(document).ready(function () {
                $('#startdateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });
                $('#enddateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });

                showhidedates();

                $('#reportperiodDropdown').change(function () {
                    showhidedates();
                });

                function showhidedates() {
                    if ($('#reportperiodDropdown').val() == 'custom')
                        $('#customdates').show();
                    else
                        $('#customdates').hide();
                }

            });
        </script>

        <div class="reporttimepicker">
            <div style="float: left; padding-right: 10px;">
                <?php echo gettext("Period"); ?>&nbsp;
                <select id='reportperiodDropdown' name="reportperiod">
                    <?php
                    $tp = get_report_timeperiod_options();
                    foreach ($tp as $shortname => $longname) {
                        echo "<option value='" . $shortname . "' " . is_selected($shortname, $reportperiod) . ">" . $longname . "</option>";
                    }
                    ?>
                </select>
            </div>

            <div id="customdates" style="float: left;">
                &nbsp;<?php echo gettext("From"); ?>&nbsp;
                <input class="textfield" type="text" id='startdateBox' name="startdate"
                       value="<?php echo $startdate; ?>" size="16"/>

                <div class="reportstartdatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="startdatepickercontainer"></div>
                &nbsp;<?php echo gettext("To"); ?>&nbsp;
                <input class="textfield" type="text" id='enddateBox' name="enddate" value="<?php echo $enddate; ?>"
                       size="16"/>

                <div class="reportenddatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="enddatepickercontainer"></div>
                &nbsp;
            </div>
        </div>


        <div class="reportoptionpicker" style="clear: left;">
            <!-- options go here... -->
            <?php echo gettext("Limit To"); ?>&nbsp;

            <select name="host" id="hostList" style="width: 150px;">
                <option value=""><?php echo gettext("Host"); ?>:</option>
                <?php
                $args = array('brevity' => 1, 'orderby' => 'host_name:a');
                $oxml = get_xml_host_objects($args);
                if ($oxml) {
                    foreach ($oxml->host as $hostobject) {
                        $name = strval($hostobject->host_name);
                        echo "<option value='" . $name . "' " . is_selected($host, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>

            <select name="hostgroup" id="hostgroupList" style="width: 150px;">
                <option value=""><?php echo gettext("Hostgroup"); ?>:</option>
                <?php
                $args = array('orderby' => 'hostgroup_name:a');
                $oxml = get_xml_hostgroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->hostgroup as $hg) {
                        $name = strval($hg->hostgroup_name);
                        echo "<option value='" . $name . "' " . is_selected($hostgroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
            <select name="servicegroup" id="servicegroupList" style="width: 150px;">
                <option value=""><?php echo gettext("Servicegroup"); ?>:</option>
                <?php
                $args = array('orderby' => 'servicegroup_name:a');
                $oxml = get_xml_servicegroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->servicegroup as $sg) {
                        $name = strval($sg->servicegroup_name);
                        echo "<option value='" . $name . "' " . is_selected($servicegroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>

            <input type='submit' class='reporttimesubmitbutton' name='reporttimesubmitbutton'
                   value='<?php echo gettext("Update"); ?>'>

            <br clear='all'>
            <a id="advanced_options_button"
               style="cursor: pointer; margin-top: 6px; display: inline-block;"><?php if (!$advanced) {
                    echo gettext("Show Advanced Options");
                } else {
                    echo gettext("Hide Advanced Options");
                } ?></a>
            <input type="hidden" value="<?php echo $advanced; ?>" id="advanced" name="advanced">

            <script type="text/javascript">
                $(document).ready(function () {
                    $('#hostList').searchable({maxMultiMatch: 9999});
                    $('#hostgroupList').searchable({maxMultiMatch: 9999});
                    $('#servicegroupList').searchable({maxMultiMatch: 9999});

                    $('#hostList').change(function () {
                        $('#hostgroupList').val('');
                        $('#servicegroupList').val('');
                    });

                    $('#servicegroupList').change(function () {
                        $('#hostList').val('');
                        $('#hostgroupList').val('');
                    });

                    $('#hostgroupList').change(function () {
                        $('#servicegroupList').val('');
                        $('#hostList').val('');
                    });

                    // Add the ability to show the advanced options section
                    $('#advanced_options_button').click(function () {
                        if ($('#advanced_options').is(":visible")) {
                            $('#advanced_options').hide();
                            $('#advanced').val(0);
                            $('#advanced_options_button').html('<?php echo gettext("Show Advanced Options"); ?>');
                        } else {
                            $('#advanced_options').show();
                            $('#advanced').val(1);
                            $('#advanced_options_button').html('<?php echo gettext("Hide Advanced Options"); ?>');
                        }
                    });

                });
            </script>

            <div id="advanced_options" style="<?php if (!$advanced) {
                echo 'display: none;';
            } ?> padding-top: 10px;">

                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("Assume Initial States"); ?>:
                    <select name="assumeinitialstates">
                        <option value="yes" <?php if ($assumeinitialstates == "yes") {
                            echo "selected";
                        } ?>><?php echo gettext("Yes"); ?></option>
                        <option value="no" <?php if ($assumeinitialstates == "no") {
                            echo "selected";
                        } ?>><?php echo gettext("No"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("Assume State Retention"); ?>:
                    <select name="assumestateretention">
                        <option value="yes" <?php if ($assumestateretention == "yes") {
                            echo "selected";
                        } ?>><?php echo gettext("Yes"); ?></option>
                        <option value="no" <?php if ($assumestateretention == "no") {
                            echo "selected";
                        } ?>><?php echo gettext("No"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("Assume States During Program Downtime"); ?>:
                    <select name="assumestatesduringdowntime">
                        <option value="yes" <?php if ($assumestatesduringdowntime == "yes") {
                            echo "selected";
                        } ?>><?php echo gettext("Yes"); ?></option>
                        <option value="no" <?php if ($assumestatesduringdowntime == "no") {
                            echo "selected";
                        } ?>><?php echo gettext("No"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("Include Soft States"); ?>:
                    <select name="includesoftstates">
                        <option value="no" <?php if ($includesoftstates == "no") {
                            echo "selected";
                        } ?>><?php echo gettext("No"); ?></option>
                        <option value="yes" <?php if ($includesoftstates == "yes") {
                            echo "selected";
                        } ?>><?php echo gettext("Yes"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("First Assumed Host State"); ?>:
                    <select name="assumedhoststate">
                        <option value="0" <?php if ($assumedhoststate == 0) {
                            echo "selected";
                        } ?>><?php echo gettext("Unspecified"); ?></option>
                        <option value="-1" <?php if ($assumedhoststate == -1) {
                            echo "selected";
                        } ?>><?php echo gettext("Current State"); ?></option>
                        <option value="3" <?php if ($assumedhoststate == 3) {
                            echo "selected";
                        } ?>><?php echo gettext("Host Up"); ?></option>
                        <option value="4" <?php if ($assumedhoststate == 4) {
                            echo "selected";
                        } ?>><?php echo gettext("Host Down"); ?></option>
                        <option value="5" <?php if ($assumedhoststate == 5) {
                            echo "selected";
                        } ?>><?php echo gettext("Host Unreachable"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("First Assumed Service State"); ?>:
                    <select name="assumedservicestate">
                        <option value="0" <?php if ($assumedservicestate == 0) {
                            echo "selected";
                        } ?>><?php echo gettext("Unspecified"); ?></option>
                        <option value="-1" <?php if ($assumedservicestate == -1) {
                            echo "selected";
                        } ?>><?php echo gettext("Current State"); ?></option>
                        <option value="6" <?php if ($assumedservicestate == 6) {
                            echo "selected";
                        } ?>><?php echo gettext("Service Ok"); ?></option>
                        <option value="7" <?php if ($assumedservicestate == 7) {
                            echo "selected";
                        } ?>><?php echo gettext("Service Warning"); ?></option>
                        <option value="8" <?php if ($assumedservicestate == 8) {
                            echo "selected";
                        } ?>><?php echo gettext("Service Unknown"); ?></option>
                        <option value="9" <?php if ($assumedservicestate == 9) {
                            echo "selected";
                        } ?>><?php echo gettext("Service Critical"); ?></option>
                    </select>
                </div>
                <div style="float: left; margin-right: 10px; padding-bottom: 10px;">
                    <?php echo gettext("Report Time Period"); ?>:
                    <select name="timeperiod" style="width: 150px;">
                        <option value="" <?php if ($timeperiod == "") {
                            echo "selected";
                        } ?>><?php echo gettext("None"); ?></option>
                        <?php
                        // Get a list of timeperiods
                        $request = array("objecttype_id" => 9);
                        $objects = new SimpleXMLElement(get_objects_xml_output($request));
                        foreach ($objects as $object) {
                            $tp = (string)$object->name1;
                            if (!empty($tp)) {
                                echo "<option " . is_selected($timeperiod, $tp) . ">" . $tp . "</option>";
                            }
                        }
                        ?>
                    </select>
                </div>

                <div style="float: left;">
                    <label><?php echo gettext("Display service performance graphs"); ?> <input type="checkbox" value="1"
                                                                                               name="servicegraphs" <?php echo is_checked($display_service_graphs, 1); ?>></label>
                </div>

                <div style="float: left; padding-left: 10px;">
                    <label
                        title="<?php echo gettext('This will count any state during scheduled downtime as OK for the SLA report'); ?>">
                        <?php echo gettext("Hide downtime"); ?> <input type="checkbox" name="dont_count_downtime"
                                                                       value="1" <?php if ($dont_count_downtime) {
                            echo "checked";
                        } ?>>
                    </label>
                </div>

                <!-- Set a variable to let us know it's okay to run this -->
                <input type="hidden" name="manual_run" value="1">

                <div style="clear: both;"></div>

            </div>

        </div>

        <?php
        // Die right here if we don't want to auto-load the page
        $disable_report_auto_run = get_option("disable_report_auto_run", 0);
        if ($disable_report_auto_run == 1 && $manual_run == 0) {
            die();
        }
        ?>

        <div style="clear: left;">
            <?php echo gettext("Report Covers"); ?>&nbsp;<?php echo gettext("From"); ?>:
            <b><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b> <?php echo gettext("To"); ?>
            : <b><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b>
        </div>

    <?php
    } //options can be suppressed for scheduled reports
    ?>

    <div id='availabilityreport' class="availabilityreport">
    <?php
    /*
    $args=array(
        "host" => $host,
        "service" => $service,
        "hostgroup" => $hostgroup,
        "servicegroup" => $servicegroup,
        "starttime" => $starttime,
        "endtime" => $endtime,
        );
    get_availability_data("host",$args,$hostdata);
    echo "HOSTS<BR>";
    print_r($hostdata);

    get_availability_data("service",$args,$servicedata);
    echo "SERVICES<BR>";
    print_r($servicedata);
*/


    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC SERVICE
    ///////////////////////////////////////////////////////////////////////////
    if ($service != "") {

        // get service availability
        $args = array(
            "host" => $host,
            "service" => $service,
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("service", $args, $servicedata);

        // check if we have data
        $have_data = false;
        if ($servicedata && intval($servicedata->havedata) == 1)
            $have_data = true;
        if ($have_data == false) {
            echo "<p>" . gettext("Availability data is not available when monitoring engine is not running") . "</p>";
        } // we have data..
        else {

            $service_ok = 0;
            $service_warning = 0;
            $service_unknown = 0;
            $service_critical = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    if (!$dont_count_downtime) {
                        $service_ok = floatval($s->percent_known_time_ok);
                        $service_warning = floatval($s->percent_known_time_warning);
                        $service_unknown = floatval($s->percent_known_time_unknown);
                        $service_critical = floatval($s->percent_known_time_critical);
                    } else {
                        $service_ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $service_warning = floatval($s->percent_known_time_warning_unscheduled);
                        $service_unknown = floatval($s->percent_known_time_unkown_unscheduled);
                        $service_critical = floatval($s->percent_known_time_critical_unscheduled);
                    }
                }
            }

            // service chart
            $service_availability = gettext("Service Availability");
            $url = "availability.php?mode=getchart&title=" . urlencode($service_availability) . "&data=" . $service_ok . "," . $service_warning . "," . $service_unknown . "," . $service_critical . "&legend=Ok,Warning,Unknown,Critical&colors=" . get_avail_color("ok") . "," . get_avail_color("warning") . "," . get_avail_color("unknown") . "," . get_avail_color("critical");

            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.get(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FEFF5F', '#FFC45F', '#FF795F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Ok', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Warning', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unknown', parseFloat(v)]);
                            } else if (k == 3) {
                                formatted_data.push(['Critical', parseFloat(v)]);
                            }
                        });

                        var service_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'service_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        service_availability_chart = new Highcharts.Chart(options);

                    }, 'json');
                });
            </script>
            <div id="service_availability_chart" style="display: inline-block;"></div>
            <?php

            // service table
            if ($servicedata && !$showonlygraphs) {
                echo "<br>";
                echo "<b>" . gettext("Service Data") . "</b>";
                echo "<table class='infotable'>";
                echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("Service") . "&nbsp;</th><th>" . gettext("Ok") . "&nbsp;</th><th>" . gettext("Warning") . "&nbsp;</th><th>" . gettext("Unknown") . "&nbsp;</th><th>" . gettext("Critical") . "&nbsp;</th></tr></thead>";
                echo "<tbody>";
                $lasthost = "";
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);

                    if (!$dont_count_downtime) {
                        $ok = floatval($s->percent_known_time_ok);
                        $wa = floatval($s->percent_known_time_warning);
                        $un = floatval($s->percent_known_time_unknown);
                        $cr = floatval($s->percent_known_time_critical);
                    } else {
                        $ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $wa = floatval($s->percent_known_time_warning_unscheduled);
                        $un = floatval($s->percent_known_time_unkown_unscheduled);
                        $cr = floatval($s->percent_known_time_critical_unscheduled);
                    }

                    // newline
                    if ($lasthost != $hn && $lasthost != "") {
                        echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                    }

                    echo "<tr>";
                    if ($lasthost != $hn)
                        echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                    else
                        echo "<td>&nbsp;&nbsp;</td>";
                    echo "<td>" . $sd . "&nbsp;&nbsp;</td>";
                    echo "<td>" . $ok . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $wa . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $un . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $cr . "%&nbsp;&nbsp;</td>";
                    echo "</tr>";

                    $lasthost = $hn;
                }

                echo "</tbody>";
                echo "</table>";
            }

        }

    }

    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC HOST
    ///////////////////////////////////////////////////////////////////////////
    else if ($host != "") {

        // get host availability
        $args = array(
            "host" => $host,
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("host", $args, $hostdata);

        // getservice availability
        $args = array(
            "host" => $host,
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("service", $args, $servicedata);

        // check if we have data
        $have_data = false;
        if ($hostdata && $servicedata && intval($hostdata->havedata) == 1 && intval($servicedata->havedata) == 1)
            $have_data = true;
        if ($have_data == false) {
            echo "<p>" . gettext("Availability data is not available when monitoring engine is not running") . ".</p>";
        } // we have data..
        else {


            $host_up = 0;
            $host_down = 0;
            $host_unreachable = 0;

            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    if (!$dont_count_downtime) {
                        $host_up = floatval($h->percent_known_time_up);
                        $host_down = floatval($h->percent_known_time_down);
                        $host_unreachable = floatval($h->percent_known_time_unreachable);
                    } else {
                        $host_up = (($ok = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled)) > 100 ? 100 : $ok);
                        $host_down = floatval($h->percent_known_time_down_unscheduled);
                        $host_unreachable = floatval($h->percent_known_time_unreachable_unscheduled);
                    }
                }
            }

            // title
            //echo "<h2>Host Data</h2>";

            // host chart
            $host_availability = gettext("Host Availability");
            $url = "availability.php?mode=getchart&title=" . urlencode($host_availability) . "&data=" . $host_up . "," . $host_down . "," . $host_unreachable . "&legend=Up,Down,Unreachable&colors=" . get_avail_color("up") . "," . get_avail_color("down") . "," . get_avail_color("unreachable");

            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.getJSON(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FF795F', '#FFC45F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Up', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Down', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unreachable', parseFloat(v)]);
                            }
                        });

                        var host_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'host_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        host_availability_chart = new Highcharts.Chart(options);
                    });
                });
            </script>
            <div id="host_availability_chart" style="display: inline-block; margin-right: 10px;"></div>
            <?php


            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    if (!$dont_count_downtime) {
                        $service_ok = floatval($s->percent_known_time_ok);
                        $service_warning = floatval($s->percent_known_time_warning);
                        $service_unknown = floatval($s->percent_known_time_unknown);
                        $service_critical = floatval($s->percent_known_time_critical);
                    } else {
                        $service_ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $service_warning = floatval($s->percent_known_time_warning_unscheduled);
                        $service_unknown = floatval($s->percent_known_time_unkown_unscheduled);
                        $service_critical = floatval($s->percent_known_time_critical_unscheduled);
                    }

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }

            // title
            //echo "<h2>Service Data</h2>";

            // service chart
            $url = "availability.php?mode=getchart&title=Average+Service+Availability&data=" . $avg_service_ok . "," . $avg_service_warning . "," . $avg_service_unknown . "," . $avg_service_critical . "&legend=Ok,Warning,Unknown,Critical&colors=" . get_avail_color("ok") . "," . get_avail_color("warning") . "," . get_avail_color("unknown") . "," . get_avail_color("critical");
            // only show service chart if there are services (some percent exists)
            if (($avg_service_ok + $avg_service_warning + $avg_service_unknown + $avg_service_critical) > 0) {
                ?>
                <script type="text/javascript">
                    $(document).ready(function () {

                        var api_url = '<?php echo $url; ?>';
                        $.getJSON(api_url, function (chartdata) {

                            Highcharts.setOptions({
                                colors: ['#b2ff5f', '#FEFF5F', '#FFC45F', '#FF795F']
                            });

                            formatted_data = new Array();
                            $.each(chartdata.data, function (k, v) {
                                if (k == 0) {
                                    formatted_data.push(['Ok', parseFloat(v)]);
                                } else if (k == 1) {
                                    formatted_data.push(['Warning', parseFloat(v)]);
                                } else if (k == 2) {
                                    formatted_data.push(['Unknown', parseFloat(v)]);
                                } else if (k == 3) {
                                    formatted_data.push(['Critical', parseFloat(v)]);
                                }
                            });

                            var service_availability_chart;
                            var options = {
                                chart: {
                                    renderTo: 'service_availability_chart',
                                    width: chartdata.width,
                                    height: chartdata.height
                                },
                                credits: {
                                    enabled: false
                                },
                                title: {
                                    text: chartdata.graph_title
                                },
                                plotOptions: {
                                    pie: {
                                        size: 160,
                                        enableMouseTracking: false,
                                        animation: false,
                                        dataLabels: {
                                            enabled: true,
                                            color: '#000',
                                            connectorColor: '#000000',
                                            format: '{point.name} {point.percentage:.2f}%'
                                        }
                                    }
                                },
                                series: [
                                    {
                                        type: 'pie',
                                        data: formatted_data
                                    }
                                ]
                            }
                            service_availability_chart = new Highcharts.Chart(options);
                        });
                    });
                </script>
                <div id="service_availability_chart" style="display: inline-block; margin-right: 10px;"></div>
            <?php
            }

            // host table
            if ($hostdata && !$showonlygraphs) {
                echo "<br>";
                echo "<b>" . gettext("Host Data") . "</b>";
                echo "<table class='infotable'>";
                echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("UP") . "&nbsp;</th><th>" . gettext("Down") . "&nbsp;</th><th>" . gettext("Unreachable") . "&nbsp;</th></tr></thead>";
                echo "<tbody>";
                foreach ($hostdata->hostavailability->host as $h) {
                    /*
                    echo "HOST: ".$h->host_name."<BR>";
                    echo "UP: ".$h->percent_known_time_up."<BR>";
                    echo "DOWN: ".$h->percent_known_time_down."<BR>";
                    echo "UNREACHABLE: ".$h->percent_known_time_unreachable."<BR>";
                    */
                    $hn = strval($h->host_name);

                    if (!$dont_count_downtime) {
                        $up = floatval($h->percent_known_time_up);
                        $dn = floatval($h->percent_known_time_down);
                        $un = floatval($h->percent_known_time_unreachable);
                    } else {
                        $up = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled);
                        $dn = floatval($h->percent_known_time_down_unscheduled);
                        $un = floatval($h->percent_known_time_unreachable_unscheduled);
                    }

                    echo "<tr>";
                    echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                    echo "<td>" . $up . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $dn . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $un . "%&nbsp;&nbsp;</td>";
                    echo "</tr>";
                }
                echo "</tbody>";
                echo "</table>";
            }

            // service table
            if ($servicedata && !$showonlygraphs) {
                echo "<br>";

                echo '<div class="availability-services">
                        <div style="float: left; padding-right: 20px;">';

                echo "<b>" . gettext("Service Data") . "</b>";
                echo "<table class='infotable'>";
                echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("Service") . "&nbsp;</th><th>" . gettext("Ok") . "&nbsp;</th><th>" . gettext("Warning") . "&nbsp;</th><th>" . gettext("Unknown") . "&nbsp;</th><th>" . gettext("Critical") . "&nbsp;</th></tr></thead>";


                echo "<tbody>";
                $lasthost = "";
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);

                    if (!$dont_count_downtime) {
                        $ok = floatval($s->percent_known_time_ok);
                        $wa = floatval($s->percent_known_time_warning);
                        $un = floatval($s->percent_known_time_unknown);
                        $cr = floatval($s->percent_known_time_critical);
                    } else {
                        $ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $wa = floatval($s->percent_known_time_warning_unscheduled);
                        $un = floatval($s->percent_known_time_unkown_unscheduled);
                        $cr = floatval($s->percent_known_time_critical_unscheduled);
                    }

                    // newline
                    if ($lasthost != $hn && $lasthost != "") {
                        echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                    }

                    echo "<tr>";
                    if ($lasthost != $hn)
                        echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                    else
                        echo "<td>&nbsp;&nbsp;</td>";
                    echo "<td>" . $sd . "&nbsp;&nbsp;</td>";
                    echo "<td>" . $ok . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $wa . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $un . "%&nbsp;&nbsp;</td>";
                    echo "<td>" . $cr . "%&nbsp;&nbsp;</td>";
                    echo "</tr>";

                    $lasthost = $hn;
                }
                // averages
                echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                echo "<tr><td>&nbsp;</td><td><b>" . gettext("Average") . "</b>&nbsp;&nbsp;</td><td>" . number_format($avg_service_ok, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_warning, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_unknown, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_critical, 3) . "%&nbsp;&nbsp;</td></tr>";

                echo "</tbody>";
                echo "</table>";

                echo '</div>';

                // Loop through each service and display a perfdata graph
                if ($display_service_graphs && !$showonlygraphs) {
                    echo '<div style="float: left;">';
                    foreach ($servicedata->serviceavailability->service as $s) {

                        $host = $s->host_name;
                        $service = $s->service_description;

                        // If rendering as pdf
                        if ($hideoptions)
                            $mode = "pdf";
                        else
                            $mode = "";

                        if (pnp_chart_exists($host, $service)) {
                            echo "<div class='serviceperfgraphcontainer'>";
                            $dargs = array(
                                DASHLET_ADDTODASHBOARDTITLE => gettext("Add This Performance Graph To A Dashboard"),
                                DASHLET_ARGS => array(
                                    "host_id" => get_host_id($host),
                                    "hostname" => $host,
                                    "servicename" => $service,
                                    "startdate" => date("Y-m-d H:i", $starttime),
                                    "enddate" => date("Y-m-d H:i", $endtime),
                                    "width" => "",
                                    "height" => "",
                                    "mode" => PERFGRAPH_MODE_SERVICEDETAIL,
                                    "render_mode" => $mode),
                                DASHLET_TITLE => $host . " " . $service . " " . gettext("Performance Graph"));

                            display_dashlet("xicore_perfdata_chart", "", $dargs, DASHLET_MODE_OUTBOARD);
                            echo "</div>";
                        }
                    }
                    echo '</div>';
                }
                // End: Display Service Graphs

            }

            echo '</div>';
        }

    }

    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC HOSTGROUP OR SERVICEGROUP
    ///////////////////////////////////////////////////////////////////////////
    else if ($hostgroup != "" || $servicegroup != "") {

        //echo "STARTTIME2: $starttime<BR>";
        //echo "ENDTIME2: $endtime<BR>";

        // get host availability
        $args = array(
            "host" => "",
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        if ($hostgroup != "")
            $args["hostgroup"] = $hostgroup;
        else
            $args["servicegroup"] = $servicegroup;
        get_availability_data("host", $args, $hostdata);

        // getservice availability
        $args = array(
            "host" => "",
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        if ($hostgroup != "")
            $args["hostgroup"] = $hostgroup;
        else
            $args["servicegroup"] = $servicegroup;
        get_availability_data("service", $args, $servicedata);

        // check if we have data
        $have_data = false;
        if ($hostdata && $servicedata && intval($hostdata->havedata) == 1 && intval($servicedata->havedata) == 1)
            $have_data = true;
        if ($have_data == false) {
            echo "<p>" . gettext("Availability data is not available when monitoring engine is not running") . ".</p>";
        } // we have data..
        else {

            $avg_host_up = 0;
            $avg_host_down = 0;
            $avg_host_unreachable = 0;
            $count_host_up = 0;
            $count_host_down = 0;
            $count_host_unreachable = 0;

            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    if (!$dont_count_downtime) {
                        $host_up = floatval($h->percent_known_time_up);
                        $host_down = floatval($h->percent_known_time_down);
                        $host_unreachable = floatval($h->percent_known_time_unreachable);
                    } else {
                        $host_up = (($ok = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled)) > 100 ? 100 : $ok);
                        $host_down = floatval($h->percent_known_time_down_unscheduled);
                        $host_unreachable = floatval($h->percent_known_time_unreachable_unscheduled);
                    }

                    update_avail_avg($avg_host_up, $host_up, $count_host_up);
                    update_avail_avg($avg_host_down, $host_down, $count_host_down);
                    update_avail_avg($avg_host_unreachable, $host_unreachable, $count_host_unreachable);
                }
            }

            // title
            //echo "<h2>Host Data</h2>";

            // host chart
            $title = urlencode('Average Host Availability');
            $up = urlencode('Up');
            $down = urlencode('Down');
            $unreachable = urlencode('Unreachable');
            $url = "availability.php?mode=getchart&title=$title&data=" . $avg_host_up . "," . $avg_host_down . "," . $avg_host_unreachable . "&legend=$up,$down,$unreachable&colors=" . get_avail_color("up") . "," . get_avail_color("down") . "," . get_avail_color("unreachable");
            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.getJSON(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FF795F', '#FFC45F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Up', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Down', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unreachable', parseFloat(v)]);
                            }
                        });

                        var host_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'host_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        host_availability_chart = new Highcharts.Chart(options);
                    });
                });
            </script>
            <div id="host_availability_chart" style="display: inline-block; margin-right: 10px;"></div>
            <?php


            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    if (!$dont_count_downtime) {
                        $service_ok = floatval($s->percent_known_time_ok);
                        $service_warning = floatval($s->percent_known_time_warning);
                        $service_unknown = floatval($s->percent_known_time_unknown);
                        $service_critical = floatval($s->percent_known_time_critical);
                    } else {
                        $service_ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $service_warning = floatval($s->percent_known_time_warning_unscheduled);
                        $service_unknown = floatval($s->percent_known_time_unkown_unscheduled);
                        $service_critical = floatval($s->percent_known_time_critical_unscheduled);
                    }

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }

            // title
            //echo "<h2>Service Data</h2>";

            // service chart
            $title = urlencode('Average Service Availability');
            $ok = urlencode('Ok');
            $warning = urlencode('Warning');
            $unknown = urlencode('Unknown');
            $critical = urlencode('Critical');
            $url = "availability.php?mode=getchart&title=$title&data=" . $avg_service_ok . "," . $avg_service_warning . "," . $avg_service_unknown . "," . $avg_service_critical . "&legend=$ok,$warning,$unknown,$critical&colors=" . get_avail_color("ok") . "," . get_avail_color("warning") . "," . get_avail_color("unknown") . "," . get_avail_color("critical");
            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.get(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FEFF5F', '#FFC45F', '#FF795F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Ok', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Warning', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unknown', parseFloat(v)]);
                            } else if (k == 3) {
                                formatted_data.push(['Critical', parseFloat(v)]);
                            }
                        });

                        var service_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'service_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        service_availability_chart = new Highcharts.Chart(options);

                    }, 'json');
                });
            </script>
            <div id="service_availability_chart" style="display: inline-block;"></div>
            <?php

            /*
            echo "<BR>HOSTDATA<BR>";
            print_r($hostdata);
            echo "<BR><BR>";
            */

            // host table
            if ($hostdata && !$showonlygraphs) {
                echo "<br>";
                echo "<b>" . gettext('Host Data') . "</b>";
                echo "<table class='infotable'>";
                echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("UP") . "&nbsp;</th><th>" . gettext("Down") . "&nbsp;</th><th>" . gettext("Unreachable") . "&nbsp;</th></tr></thead>";
                echo "<tbody>";
                if ($showdetail == 1) {
                    $lasthost = "";
                    foreach ($hostdata->hostavailability->host as $h) {

                        $hn = strval($h->host_name);

                        if (!$dont_count_downtime) {
                            $up = floatval($h->percent_known_time_up);
                            $dn = floatval($h->percent_known_time_down);
                            $un = floatval($h->percent_known_time_unreachable);
                        } else {
                            $up = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled);
                            $dn = floatval($h->percent_known_time_down_unscheduled);
                            $un = floatval($h->percent_known_time_unreachable_unscheduled);
                        }

                        if ($lasthost != "" && $lasthost != $hn)
                            echo "<tr><td colspan='4'><hr noshade size='1'></td></tr>";

                        echo "<tr>";
                        echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($up, 3) . "%&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($dn, 3) . "%&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($un, 3) . "%&nbsp;&nbsp;</td>";
                        echo "</tr>";

                        $lasthost = $hn;
                    }
                }
                // averages
                if ($showdetail == 1)
                    echo "<tr><td colspan='4'><hr noshade size='1'></td></tr>";
                echo "<tr><td><b>" . gettext("Average") . "</b>&nbsp;&nbsp;</td><td>" . number_format($avg_host_up, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_host_down, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_host_unreachable, 3) . "%&nbsp;&nbsp;</td></tr>";

                echo "</tbody>";
                echo "</table>";
            }

            // service table
            if ($servicedata && !$showonlygraphs) {

                echo "<br>";

                echo '<div class="availability-services">
                        <div style="float: left; padding-right: 20px;">';


                echo "<b>" . gettext('Service Data') . "</b>";
                echo "<table class='infotable'>";

                echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("Service") . "&nbsp;</th><th>" . gettext("Ok") . "&nbsp;</th><th>" . gettext("Warning") . "&nbsp;</th><th>" . gettext("Unknown") . "&nbsp;</th><th>" . gettext("Critical") . "&nbsp;</th></tr></thead>";
                echo "<tbody>";
                if ($showdetail == 1) {
                    $lasthost = "";
                    foreach ($servicedata->serviceavailability->service as $s) {

                        $hn = strval($s->host_name);
                        $sd = strval($s->service_description);

                        if (!$dont_count_downtime) {
                            $ok = floatval($s->percent_known_time_ok);
                            $wa = floatval($s->percent_known_time_warning);
                            $un = floatval($s->percent_known_time_unknown);
                            $cr = floatval($s->percent_known_time_critical);
                        } else {
                            $ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                            $wa = floatval($s->percent_known_time_warning_unscheduled);
                            $un = floatval($s->percent_known_time_unkown_unscheduled);
                            $cr = floatval($s->percent_known_time_critical_unscheduled);
                        }

                        // newline
                        if ($lasthost != $hn && $lasthost != "") {
                            echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                        }

                        echo "<tr>";
                        if ($lasthost != $hn)
                            echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                        else
                            echo "<td>&nbsp;&nbsp;</td>";
                        echo "<td>" . $sd . "&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($ok, 3) . "%&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($wa, 3) . "%&nbsp;&nbsp;</td>";
                        echo "<td>" . number_format($un, 3) . "%&nbsp;&nbsp;</td>";
                        echo "<td>" . $cr . "%&nbsp;&nbsp;</td>";
                        echo "</tr>";

                        $lasthost = $hn;
                    }
                }
                // averages
                if ($showdetail == 1)
                    echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                echo "<tr><td>&nbsp;</td><td><b>" . gettext("Average") . "</b>&nbsp;&nbsp;</td><td>" . number_format($avg_service_ok, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_warning, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_unknown, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_critical, 3) . "%&nbsp;&nbsp;</td></tr>";

                echo "</tbody>";
                echo "</table>";

                echo '</div>';

                // Loop through each service and display a perfdata graph
                if ($display_service_graphs && !$showonlygraphs) {
                    echo '<div style="float: left;">';
                    foreach ($servicedata->serviceavailability->service as $s) {

                        $host = $s->host_name;
                        $service = $s->service_description;

                        // If rendering as pdf
                        if ($hideoptions)
                            $mode = "pdf";
                        else
                            $mode = "";

                        if (pnp_chart_exists($host, $service)) {
                            echo "<div class='serviceperfgraphcontainer'>";
                            $dargs = array(
                                DASHLET_ADDTODASHBOARDTITLE => gettext("Add This Performance Graph To A Dashboard"),
                                DASHLET_ARGS => array(
                                    "host_id" => get_host_id($host),
                                    "hostname" => $host,
                                    "servicename" => $service,
                                    "startdate" => date("Y-m-d H:i", $starttime),
                                    "enddate" => date("Y-m-d H:i", $endtime),
                                    "width" => "",
                                    "height" => "",
                                    "mode" => PERFGRAPH_MODE_SERVICEDETAIL,
                                    "render_mode" => $mode),
                                DASHLET_TITLE => $host . " " . $service . " " . gettext("Performance Graph"));

                            display_dashlet("xicore_perfdata_chart", "", $dargs, DASHLET_MODE_OUTBOARD);
                            echo "</div>";
                        }
                    }
                    echo '</div>';
                }
                // End: Display Service Graphs

                echo '</div>';
            }

        }

    }


    ///////////////////////////////////////////////////////////////////////////
    // OVERVIEW (ALL HOSTS AND SERVICES)
    ///////////////////////////////////////////////////////////////////////////
    else {

        // get host availability
        $args = array(
            "host" => "all",
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("host", $args, $hostdata);

        // getservice availability
        $args = array(
            "host" => "all",
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("service", $args, $servicedata);

        // check if we have data
        $have_data = false;
        if ($hostdata && $servicedata && intval($hostdata->havedata) == 1 && intval($servicedata->havedata) == 1)
            $have_data = true;
        if ($have_data == false) {
            echo "<p>" . gettext("Availability data is not available when monitoring engine is not running") . ".</p>";
        } // we have data..
        else {

            $avg_host_up = 0;
            $avg_host_down = 0;
            $avg_host_unreachable = 0;
            $count_host_up = 0;
            $count_host_down = 0;
            $count_host_unreachable = 0;

            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    if (!$dont_count_downtime) {
                        $host_up = floatval($h->percent_known_time_up);
                        $host_down = floatval($h->percent_known_time_down);
                        $host_unreachable = floatval($h->percent_known_time_unreachable);
                    } else {
                        $host_up = (($ok = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled)) > 100 ? 100 : $ok);
                        $host_down = floatval($h->percent_known_time_down_unscheduled);
                        $host_unreachable = floatval($h->percent_known_time_unreachable_unscheduled);
                    }

                    update_avail_avg($avg_host_up, $host_up, $count_host_up);
                    update_avail_avg($avg_host_down, $host_down, $count_host_down);
                    update_avail_avg($avg_host_unreachable, $host_unreachable, $count_host_unreachable);
                }
            }

            // title
            //echo "<h2>Host Data</h2>";

            // host chart
            $title = urlencode('Average Host Availability');
            $up = urlencode('Up');
            $down = urlencode('Down');
            $unreachable = urlencode('Unreachable');
            $url = "availability.php?mode=getchart&type=host&title=$title&data=" . $avg_host_up . "," . $avg_host_down . "," . $avg_host_unreachable . "&legend=$up,$down,$unreachable&colors=" . get_avail_color("up") . "," . get_avail_color("down") . "," . get_avail_color("unreachable");

            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.getJSON(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FF795F', '#FFC45F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Up', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Down', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unreachable', parseFloat(v)]);
                            }
                        });

                        var host_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'host_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        host_availability_chart = new Highcharts.Chart(options);
                    });
                });
            </script>
            <div id="host_availability_chart" style="display: inline-block; margin-right: 10px;"></div>
            <?php

            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    if (!$dont_count_downtime) {
                        $service_ok = floatval($s->percent_known_time_ok);
                        $service_warning = floatval($s->percent_known_time_warning);
                        $service_unknown = floatval($s->percent_known_time_unknown);
                        $service_critical = floatval($s->percent_known_time_critical);
                    } else {
                        $service_ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                        $service_warning = floatval($s->percent_known_time_warning_unscheduled);
                        $service_unknown = floatval($s->percent_known_time_unkown_unscheduled);
                        $service_critical = floatval($s->percent_known_time_critical_unscheduled);
                    }

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }

            // title
            //echo "<h2>Service Data</h2>";

            // service chart
            $title = urlencode('Average Service Availability');
            $ok = urlencode('Ok');
            $warning = urlencode('Warning');
            $unknown = urlencode('Unknown');
            $critical = urlencode('Critical');
            $url = "availability.php?mode=getchart&type=service&title=$title&data=" . $avg_service_ok . "," . $avg_service_warning . "," . $avg_service_unknown . "," . $avg_service_critical . "&legend=$ok,$warning,$unknown,$critical&colors=" . get_avail_color("ok") . "," . get_avail_color("warning") . "," . get_avail_color("unknown") . "," . get_avail_color("critical");

            ?>
            <script type="text/javascript">
                $(document).ready(function () {

                    var api_url = '<?php echo $url; ?>';
                    $.get(api_url, function (chartdata) {

                        Highcharts.setOptions({
                            colors: ['#b2ff5f', '#FEFF5F', '#FFC45F', '#FF795F']
                        });

                        formatted_data = new Array();
                        $.each(chartdata.data, function (k, v) {
                            if (k == 0) {
                                formatted_data.push(['Ok', parseFloat(v)]);
                            } else if (k == 1) {
                                formatted_data.push(['Warning', parseFloat(v)]);
                            } else if (k == 2) {
                                formatted_data.push(['Unknown', parseFloat(v)]);
                            } else if (k == 3) {
                                formatted_data.push(['Critical', parseFloat(v)]);
                            }
                        });

                        var service_availability_chart;
                        var options = {
                            chart: {
                                renderTo: 'service_availability_chart',
                                width: chartdata.width,
                                height: chartdata.height
                            },
                            credits: {
                                enabled: false
                            },
                            title: {
                                text: chartdata.graph_title
                            },
                            plotOptions: {
                                pie: {
                                    size: 160,
                                    enableMouseTracking: false,
                                    animation: false,
                                    dataLabels: {
                                        enabled: true,
                                        color: '#000',
                                        connectorColor: '#000000',
                                        format: '{point.name} {point.percentage:.2f}%'
                                    }
                                }
                            },
                            series: [
                                {
                                    type: 'pie',
                                    data: formatted_data
                                }
                            ]
                        }
                        service_availability_chart = new Highcharts.Chart(options);

                    }, 'json');
                });
            </script>
            <div id="service_availability_chart" style="display: inline-block;"></div>
            <?php
            if (!$showonlygraphs) {

                // host table
                if ($hostdata && !$showonlygraphs) {
                    echo "<br>";
                    echo "<b>" . gettext('Host Data') . "</b>";
                    echo "<table class='infotable'>";
                    echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("UP") . "&nbsp;</th><th>" . gettext("Down") . "&nbsp;</th><th>" . gettext("Unreachable") . "&nbsp;</th></tr></thead>";
                    echo "<tbody>";
                    if ($showdetail == 1) {
                        $lasthost = "";
                        foreach ($hostdata->hostavailability->host as $h) {

                            $hn = strval($h->host_name);

                            if (!$dont_count_downtime) {
                                $up = floatval($h->percent_known_time_up);
                                $dn = floatval($h->percent_known_time_down);
                                $un = floatval($h->percent_known_time_unreachable);
                            } else {
                                $up = floatval($h->percent_known_time_up) + floatval($h->percent_known_time_down_scheduled) + floatval($h->percent_known_time_unreachable_scheduled);
                                $dn = floatval($h->percent_known_time_down_unscheduled);
                                $un = floatval($h->percent_known_time_unreachable_unscheduled);
                            }

                            if ($lasthost != "" && $lasthost != $hn)
                                echo "<tr><td colspan='4'><hr noshade size='1'></td></tr>";

                            echo "<tr>";
                            echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                            echo "<td>" . $up . "%&nbsp;&nbsp;</td>";
                            echo "<td>" . $dn . "%&nbsp;&nbsp;</td>";
                            echo "<td>" . $un . "%&nbsp;&nbsp;</td>";
                            echo "</tr>";

                            $lasthost = $hn;
                        }
                    }
                    // averages
                    if ($showdetail == 1)
                        echo "<tr><td colspan='4'><hr noshade size='1'></td></tr>";
                    echo "<tr><td><b>" . gettext("Average") . "</b>&nbsp;&nbsp;</td><td>" . number_format($avg_host_up, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_host_down, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_host_unreachable, 3) . "%&nbsp;&nbsp;</td></tr>";

                    echo "</tbody>";
                    echo "</table>";
                }

                // service table
                if ($servicedata && !$showonlygraphs) {
                    echo "<br>";
                    echo "<b>" . gettext("Service Data") . "</b>";
                    echo "<table class='infotable'>";

                    echo "<thead><tr><th>" . gettext("Host") . "&nbsp;</th><th>" . gettext("Service") . "&nbsp;</th><th>" . gettext("Ok") . "&nbsp;</th><th>" . gettext("Warning") . "&nbsp;</th><th>" . gettext("Unknown") . "&nbsp;</th><th>" . gettext("Critical") . "&nbsp;</th></tr></thead>";
                    echo "<tbody>";
                    if ($showdetail == 1) {
                        $lasthost = "";
                        foreach ($servicedata->serviceavailability->service as $s) {

                            $hn = strval($s->host_name);
                            $sd = strval($s->service_description);

                            if (!$dont_count_downtime) {
                                $ok = floatval($s->percent_known_time_ok);
                                $wa = floatval($s->percent_known_time_warning);
                                $un = floatval($s->percent_known_time_unknown);
                                $cr = floatval($s->percent_known_time_critical);
                            } else {
                                $ok = (($ok = floatval($s->percent_known_time_ok) + floatval($s->percent_known_time_warning_scheduled) + floatval($s->percent_known_time_critical_scheduled) + floatval($s->percent_known_time_unkown_scheduled)) > 100 ? 100 : $ok);
                                $wa = floatval($s->percent_known_time_warning_unscheduled);
                                $un = floatval($s->percent_known_time_unkown_unscheduled);
                                $cr = floatval($s->percent_known_time_critical_unscheduled);
                            }

                            // newline
                            if ($lasthost != $hn && $lasthost != "") {
                                echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                            }

                            echo "<tr>";
                            if ($lasthost != $hn)
                                echo "<td>" . $hn . "&nbsp;&nbsp;</td>";
                            else
                                echo "<td>&nbsp;&nbsp;</td>";
                            echo "<td>" . $sd . "&nbsp;&nbsp;</td>";
                            echo "<td>" . $ok . "%&nbsp;&nbsp;</td>";
                            echo "<td>" . $wa . "%&nbsp;&nbsp;</td>";
                            echo "<td>" . $un . "%&nbsp;&nbsp;</td>";
                            echo "<td>" . $cr . "%&nbsp;&nbsp;</td>";
                            echo "</tr>";

                            $lasthost = $hn;
                        }
                    }
                    // averages
                    if ($showdetail == 1)
                        echo "<tr><td colspan='6'><hr noshade size='1'></td></tr>";
                    echo "<tr><td>&nbsp;</td><td><b>" . gettext("Average") . "</b>&nbsp;&nbsp;</td><td>" . number_format($avg_service_ok, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_warning, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_unknown, 3) . "%&nbsp;&nbsp;</td><td>" . number_format($avg_service_critical, 3) . "%&nbsp;&nbsp;</td></tr>";

                    echo "</tbody>";
                    echo "</table>";
                }
            }

        } // end show only graphs
    }
    ?>
    </div>
    <?php
    // closes the HTML page
    if (!$showonlygraphs) {
        do_page_end(true);
    }
}

/**
 * @param $apct
 * @param $npct
 * @param $cnt
 */
function update_avail_avg(&$apct, $npct, &$cnt)
{

    $newpct = (($apct * $cnt) + $npct) / ($cnt + 1);

    $cnt++;

    $apct = $newpct;
}

/**
 * @param $state
 *
 * @return string
 */
function get_avail_color($state)
{
    switch ($state) {
        case "up":
        case "ok":
            $c = "56DA56";
            break;
        case "down":
            $c = "E9513D";
            break;
        case "unreachable":
            $c = "CB2525";
            break;
        case "warning":
            $c = "F6EB3A";
            break;
        case "critical":
            $c = "F35F3D";
            break;
        case "unknown":
            $c = "F3AC3D";
            break;
        default:
            $c = "000000";
            break;
    }
    return "%23" . $c;
}

function get_availability_csv()
{

    // get values passed in GET/POST request
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");

    $host = grab_request_var("host", "");
    $service = grab_request_var("service", "");
    $hostgroup = grab_request_var("hostgroup", "");
    $servicegroup = grab_request_var("servicegroup", "");

    $csvtype = grab_request_var("csvtype", "service");

    // determine start/end times based on period
    get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

    // ADVANCED OPTIONS
    $timeperiod = grab_request_var("timeperiod", "");
    $assumeinitialstates = grab_request_var("assumeinitialstates", "yes");
    $assumestateretention = grab_request_var("assumestateretention", "yes");
    $assumestatesduringdowntime = grab_request_var("assumestatesduringdowntime", "yes");
    $includesoftstates = grab_request_var("includesoftstates", "no");
    $assumedhoststate = grab_request_var("assumedhoststate", 3);
    $assumedservicestate = grab_request_var("assumedservicestate", 6);

    header("Content-type: application/octet-stream");
    header("Content-Disposition: attachment; filename=\"availability.csv\"");
    write_csv_header($csvtype);

    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC SERVICE
    ///////////////////////////////////////////////////////////////////////////
    if ($service != "") {

        // get service availability
        $args = array(
            "host" => $host,
            "service" => $service,
            "starttime" => $starttime,
            "endtime" => $endtime,
            "timeperiod" => $timeperiod,
            "assume_initial_states" => $assumeinitialstates,
            "assume_state_retention" => $assumestateretention,
            "assume_states_during_not_running" => $assumestatesduringdowntime,
            "include_soft_states" => $includesoftstates,
            "initial_assumed_host_state" => $assumedhoststate,
            "initial_assumed_service_state" => $assumedservicestate
        );
        get_availability_data("service", $args, $servicedata);

        // check if we have data
        $have_data = false;
        if ($servicedata && intval($servicedata->havedata) == 1)
            $have_data = true;
        if ($have_data == false) {
            echo gettext("Availability data is not available when monitoring engine is not running") . ".\n";
        } // we have data..
        else {

            // service table
            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);
                    $ok = floatval($s->percent_known_time_ok);
                    $wa = floatval($s->percent_known_time_warning);
                    $un = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    write_service_csv_data($hn, $sd, $ok, $wa, $un, $service_critical);
                }
            }

        }

    }

    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC HOST
    ///////////////////////////////////////////////////////////////////////////
    else if ($host != "") {

        // get host availability
        if ($csvtype == "host") {
            $args = array(
                "host" => $host,
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            get_availability_data("host", $args, $hostdata);
        } // getservice availability
        else {
            $args = array(
                "host" => $host,
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            get_availability_data("service", $args, $servicedata);
        }

        // check if we have data
        $have_data = false;
        if (($csvtype == "host" && $hostdata && intval($hostdata->havedata) == 1) || ($servicedata && intval($servicedata->havedata) == 1))
            $have_data = true;
        if ($have_data == false) {
            echo gettext("Availability data is not available when monitoring engine is not running") . ".\n";
        } // we have data..
        else {

            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    $service_ok = floatval($s->percent_known_time_ok);
                    $service_warning = floatval($s->percent_known_time_warning);
                    $service_unknown = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }


            // host table
            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    $hn = strval($h->host_name);
                    $up = floatval($h->percent_known_time_up);
                    $dn = floatval($h->percent_known_time_down);
                    $un = floatval($h->percent_known_time_unreachable);

                    write_host_csv_data($hn, $up, $dn, $un);
                }
            }

            // service table
            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);
                    $ok = floatval($s->percent_known_time_ok);
                    $wa = floatval($s->percent_known_time_warning);
                    $un = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    write_service_csv_data($hn, $sd, $ok, $wa, $un, $service_critical);
                }
                // averages
                write_service_csv_data("", "AVERAGE", $avg_service_ok, $avg_service_warning, $avg_service_unknown, $avg_service_critical);
            }
        }

    }

    ///////////////////////////////////////////////////////////////////////////
    // SPECIFIC HOSTGROUP OR SERVICEGROUP
    ///////////////////////////////////////////////////////////////////////////
    else if ($hostgroup != "" || $servicegroup != "") {

        // get host availability
        if ($csvtype == "host") {
            $args = array(
                "host" => "",
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            if ($hostgroup != "")
                $args["hostgroup"] = $hostgroup;
            else
                $args["servicegroup"] = $servicegroup;
            get_availability_data("host", $args, $hostdata);
        } // getservice availability
        else {
            $args = array(
                "host" => "",
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            if ($hostgroup != "")
                $args["hostgroup"] = $hostgroup;
            else
                $args["servicegroup"] = $servicegroup;
            get_availability_data("service", $args, $servicedata);
        }

        // check if we have data
        $have_data = false;
        if (($csvtype == "host" && $hostdata && intval($hostdata->havedata) == 1) || ($servicedata && intval($servicedata->havedata) == 1))
            $have_data = true;
        if ($have_data == false) {
            echo gettext("Availability data is not available when monitoring engine is not running") . ".\n";
        } // we have data..
        else {

            $avg_host_up = 0;
            $avg_host_down = 0;
            $avg_host_unreachable = 0;
            $count_host_up = 0;
            $count_host_down = 0;
            $count_host_unreachable = 0;

            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    $host_up = floatval($h->percent_known_time_up);
                    $host_down = floatval($h->percent_known_time_down);
                    $host_unreachable = floatval($h->percent_known_time_unreachable);

                    update_avail_avg($avg_host_up, $host_up, $count_host_up);
                    update_avail_avg($avg_host_down, $host_down, $count_host_down);
                    update_avail_avg($avg_host_unreachable, $host_unreachable, $count_host_unreachable);
                }
            }


            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    $service_ok = floatval($s->percent_known_time_ok);
                    $service_warning = floatval($s->percent_known_time_warning);
                    $service_unknown = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }

            // host table
            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {

                    $hn = strval($h->host_name);
                    $up = floatval($h->percent_known_time_up);
                    $dn = floatval($h->percent_known_time_down);
                    $un = floatval($h->percent_known_time_unreachable);

                    write_host_csv_data($hn, $up, $dn, $un);
                }
                // averages
                write_host_csv_data("AVERAGE", $avg_host_up, $avg_host_down, $avg_host_unreachable);
            }

            // service table
            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);
                    $ok = floatval($s->percent_known_time_ok);
                    $wa = floatval($s->percent_known_time_warning);
                    $un = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    write_service_csv_data($hn, $sd, $ok, $wa, $un, $service_critical);
                }
                // averages
                write_service_csv_data("", "AVERAGE", $avg_service_ok, $avg_service_warning, $avg_service_unknown, $avg_service_critical);
            }
        }

    }


    ///////////////////////////////////////////////////////////////////////////
    // OVERVIEW (ALL HOSTS AND SERVICES)
    ///////////////////////////////////////////////////////////////////////////
    else {

        // get host availability
        if ($csvtype == "host") {
            $args = array(
                "host" => "all",
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            get_availability_data("host", $args, $hostdata);
        } // getservice availability
        else {
            $args = array(
                "host" => "all",
                "starttime" => $starttime,
                "endtime" => $endtime,
                "timeperiod" => $timeperiod,
                "assume_initial_states" => $assumeinitialstates,
                "assume_state_retention" => $assumestateretention,
                "assume_states_during_not_running" => $assumestatesduringdowntime,
                "include_soft_states" => $includesoftstates,
                "initial_assumed_host_state" => $assumedhoststate,
                "initial_assumed_service_state" => $assumedservicestate
            );
            get_availability_data("service", $args, $servicedata);
        }

        // check if we have data
        $have_data = false;
        if (($csvtype == "host" && $hostdata && intval($hostdata->havedata) == 1) || ($servicedata && intval($servicedata->havedata) == 1))
            $have_data = true;
        if ($have_data == false) {
            echo gettext("Availability data is not available when monitoring engine is not running") . ".\n";
        } // we have data..
        else {

            $avg_host_up = 0;
            $avg_host_down = 0;
            $avg_host_unreachable = 0;
            $count_host_up = 0;
            $count_host_down = 0;
            $count_host_unreachable = 0;

            if ($hostdata) {
                foreach ($hostdata->hostavailability->host as $h) {
                    $host_up = floatval($h->percent_known_time_up);
                    $host_down = floatval($h->percent_known_time_down);
                    $host_unreachable = floatval($h->percent_known_time_unreachable);

                    update_avail_avg($avg_host_up, $host_up, $count_host_up);
                    update_avail_avg($avg_host_down, $host_down, $count_host_down);
                    update_avail_avg($avg_host_unreachable, $host_unreachable, $count_host_unreachable);
                }
            }

            $avg_service_ok = 0;
            $avg_service_warning = 0;
            $avg_service_unknown = 0;
            $avg_service_critical = 0;
            $count_service_critical = 0;
            $count_service_warning = 0;
            $count_service_unknown = 0;

            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {
                    $service_ok = floatval($s->percent_known_time_ok);
                    $service_warning = floatval($s->percent_known_time_warning);
                    $service_unknown = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    update_avail_avg($avg_service_ok, $service_ok, $count_service_ok);
                    update_avail_avg($avg_service_warning, $service_warning, $count_service_warning);
                    update_avail_avg($avg_service_unknown, $service_unknown, $count_service_unknown);
                    update_avail_avg($avg_service_critical, $service_critical, $count_service_critical);
                }
            }

            // host table
            if ($hostdata) {

                foreach ($hostdata->hostavailability->host as $h) {

                    $hn = strval($h->host_name);
                    $up = floatval($h->percent_known_time_up);
                    $dn = floatval($h->percent_known_time_down);
                    $un = floatval($h->percent_known_time_unreachable);

                    write_host_csv_data($hn, $up, $dn, $un);
                }
                // averages
                write_host_csv_data("AVERAGE", $avg_host_up, $avg_host_down, $avg_host_unreachable);
            }

            // service table
            if ($servicedata) {
                foreach ($servicedata->serviceavailability->service as $s) {

                    $hn = strval($s->host_name);
                    $sd = strval($s->service_description);
                    $ok = floatval($s->percent_known_time_ok);
                    $wa = floatval($s->percent_known_time_warning);
                    $un = floatval($s->percent_known_time_unknown);
                    $service_critical = floatval($s->percent_known_time_critical);

                    write_service_csv_data($hn, $sd, $ok, $wa, $un, $service_critical);
                }
                // averages
                write_service_csv_data("", "AVERAGE", $avg_service_ok, $avg_service_warning, $avg_service_unknown, $avg_service_critical);
            }
        }


    }


}


/**
 * @param $csvtype
 */
function write_csv_header($csvtype)
{
    if ($csvtype == "service")
        echo "host,service,ok %,warning %,unknown %,critical %\n";
    else
        echo "host,up %,down %,unreachable %\n";
}

/**
 * @param $hn
 * @param $up
 * @param $dn
 * @param $un
 */
function write_host_csv_data($hn, $up, $dn, $un)
{
    echo "\"$hn\",$up,$dn,$un\n";
}

/**
 * @param $hn
 * @param $sn
 * @param $ok
 * @param $wa
 * @param $un
 * @param $cr
 */
function write_service_csv_data($hn, $sn, $ok, $wa, $un, $cr)
{
    echo "\"$hn\",\"$sn\",$ok,$wa,$un,$cr\n";
}


function get_availability_pdf()
{
    global $cfg;

    //get user name
    $username = $_SESSION["username"];
    // get backend ticket
    $backend_ticket = get_user_attr(0, "backend_ticket");

    // Assemble actual URL that will be gotten
    $prefix = ($cfg['use_https'] == true) ? 'https' : 'http';
    $uri = str_replace("mode=pdf", "hideoptions=1", $_SERVER["REQUEST_URI"]);
    $fullurl = "{$prefix}://127.0.0.1{$uri}";
    //echo "FULLURL: $fullurl\n";

    $urlparts = parse_url($fullurl);

    $newurl = "";
    $newurl .= get_internal_url() . "/reports/availability.php";
    $newurl .= "?";
    if (isset($urlparts['query']))
        $newurl .= $urlparts['query'];

    $newurl .= "&username=" . $username;
    $newurl .= "&ticket=" . $backend_ticket;

    //echo "NEWURL: $newurl\n";

    // Add language to url
    $language = $_SESSION['language'];
    $newurl .= "&locale=" . $language;

    // Do page rendering

    $aurl = $newurl;

    $afile = "page.pdf";
    $fname = get_tmp_dir() . "/scheduledreport-" . $username . "-" . time() . "-" . $afile;

    $cmdft = '--footer-spacing 3 --margin-bottom 15mm --footer-font-size 9 --footer-right "Page [page] of [toPage]" --footer-left "' . get_datetime_string(time(), DT_SHORT_DATE_TIME, DF_AUTO, "null") . '"';

    $cmd = "/usr/bin/wkhtmltopdf --no-outline {$cmdft} '{$aurl}' '{$fname}' 2>&1";
    @exec($cmd);

    if (!file_exists($fname)) {
        echo "\n\n************\nERROR: Failed to render URL '" . $aurl . "' as '" . $fname . "'\n************\n\n";
    } else {
        // We'll be outputting a PDF
        header('Content-type: application/pdf');
        header('Content-Disposition: inline; filename="availability.pdf"');

        // The PDF source is in original.pdf
        readfile($fname);
        unlink($fname);
    }
}

?>