<?php
//
// Alert Histogram Report
// Copyright (c) 2010-2014 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// Initialization stuff
pre_init();
init_session();
grab_request_vars();

// Check prereqs and authentication
check_prereqs();
check_authentication(false);

route_request();

function route_request()
{
    $mode = grab_request_var("mode", "");
    switch ($mode) {
        case "image":
            get_histogram_image();
            break;
        case "csv":
            get_histogram_csv();
            break;
        case "pdf":
            get_histogram_pdf();
            break;
        default:
            display_histogram();
            break;
    }
}

///////////////////////////////////////////////////////////////////
// BACKEND DATA FUNCTIONS
///////////////////////////////////////////////////////////////////

/**
 * Gets state history data in XML format from the backend.
 *
 * @param $args
 *
 * @return SimpleXMLElement
 */
function get_histogram_data($args)
{
    $xml = get_xml_histogram($args);
    return $xml;
}

///////////////////////////////////////////////////////////////////
// REPORT GENERATION FUCNTIONS
///////////////////////////////////////////////////////////////////

// Displays the data by generating the HTML page
function display_histogram()
{
    global $lstr;
    global $request;

    // Makes sure user has appropriate license level
    licensed_feature_check();

    // Get values passed in GET/POST request
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");
    $search = grab_request_var("search", "");
    $host = grab_request_var("host", "");
    $service = grab_request_var("service", "");
    $hostgroup = grab_request_var("hostgroup", "");
    $servicegroup = grab_request_var("servicegroup", "");
    $groupby = grab_request_var("groupby", "hour_of_day");
    $statetype = grab_request_var("statetype", "both");
    $hideoptions = grab_request_var("hideoptions", 0);
    $manual_run = grab_request_var("manual_run", 0);

    // Do not do any processing unless we have default report running enabled
    $disable_report_auto_run = get_option("disable_report_auto_run", 0);
    if ($manual_run == 1 || $disable_report_auto_run == 0) {

        // Fix search
        if ($search == $lstr['SearchBoxText']) {
            $search = "";
        }

        // Fix custom dates
        if ($reportperiod == "custom") {
            if ($enddate == "") {
                $enddate = strftime("%c", time());
            }
            if ($startdate == "") {
                $startdate = strftime("%c", time() - (60 * 60 * 24));
                $enddate = strftime("%c", time());
            }
        }

        // special "all" stuff
        if ($hostgroup == "all")
            $hostgroup = "";
        if ($servicegroup == "all")
            $servicegroup = "";
        if ($host == "all")
            $host = "";

        // can do hostgroup OR servicegroup OR host
        if ($hostgroup != "") {
            $servicegroup = "";
            $host = "";
        } else if ($servicegroup != "") {
            $host = "";
        }

        //  limit hosts by hostgroup or host
        $host_ids = array();
        //  limit by hostgroup
        if ($hostgroup != "") {
            $host_ids = get_hostgroup_member_ids($hostgroup);
        }
        //  limit by host
        //else if($host!=""){
        ///	$host_ids[]=get_host_id($host);
        //	}
        //  limit service by servicegroup
        $service_ids = array();
        if ($servicegroup != "") {
            $service_ids = get_servicegroup_member_ids($servicegroup);
        }

        $object_ids_str = "";
        $y = 0;
        foreach ($host_ids as $hid) {
            if ($y > 0)
                $object_ids_str .= ",";
            $object_ids_str .= $hid;
            $y++;
        }
        foreach ($service_ids as $sid) {
            if ($y > 0)
                $object_ids_str .= ",";
            $object_ids_str .= $sid;
            $y++;
        }

        // start/end times must be in unix timestamp format
        // if they weren't specified, default to last 24 hours
        //$endtime=grab_request_var("endtime",time());
        //$starttime=grab_request_var("starttime",$endtime-(24*60*60));

        // determine start/end times based on period
        get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

        /**/
        // SPECIFIC RECORDS (FOR PAGING): if you want to get specific records, use this type of format:
        /**/
        $args = array(
            "starttime" => $starttime,
            "endtime" => $endtime,
            "groupby" => $groupby,
        );
        //if($search)
//		$args["output"]="lk:".$search;
        // object id limiters
        if ($object_ids_str != "")
            $args["object_id"] = "in:" . $object_ids_str;
        else {
            if ($host != "") {
                $args["host_name"] = $host;
                $args["objecttype_id"] = OBJECTTYPE_HOST;
            }
            if ($service != "") {
                $args["service_description"] = $service;
                $args["objecttype_id"] = OBJECTTYPE_SERVICE;
            }
        }

        //$xml=get_histogram_data($args);
        //$xml=null;
        /**/
        //print_r($xml);

    } // End DISABLED LOADING

    // Start the HTML page
    do_page_start(array("page_title" => gettext("Alert Histogram")), true);

    // Show only the options required if not rendering
    if ($hideoptions) {

        // default logo stuff
        $logo = "nagiosxi-logo-small.png";
        $logo_alt = get_product_name();

        // use custom logo if it exists
        $logosettings_raw = get_option("custom_logo_options");
        if ($logosettings_raw == "") {
            $logosettings = array();
        } else {
            $logosettings = unserialize($logosettings_raw);
        }

        $custom_logo_enabled = grab_array_var($logosettings, "enabled");
        if ($custom_logo_enabled == 1) {
            $logo = grab_array_var($logosettings, "logo", $logo);
            $logo_alt = grab_array_var($logosettings, "logo_alt", $logo_alt);
        }
        ?>

        <div style="padding-bottom: 10px;">
            <div style="float: left; margin-right: 30px;">
                <img src="<?php echo get_base_url(); ?>images/<?php echo $logo; ?>" border="0"
                     alt="<?php echo $logo_alt; ?>" title="<?php echo $logo_alt; ?>">
            </div>
            <div style="float: left; height: 44px;">
                <div
                    style="font-weight: bold; font-size: 22px; padding-bottom: 4px;"><?php echo gettext("Alert Histogram"); ?></div>
                <div><?php echo gettext("From"); ?>:
                    <strong><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong> <?php echo gettext("To"); ?>
                    : <strong><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong>
                </div>
            </div>
            <div style="clear:both;"></div>
        </div>

        <?php if (!empty($service)) {
            ?>
            <div class="servicestatusdetailheader">
                <div class="serviceimage">
                    <!--image-->
                    <?php show_object_icon($host, $service, true); ?>
                </div>
                <div class="servicetitle">
                    <div class="servicename"><?php echo htmlentities($service); ?></div>
                    <div class="hostname"><?php echo htmlentities($host); ?></div>
                </div>
            </div>
            <div class="clear"></div>
        <?php } else if (!empty($host) || $service == '*') { ?>
            <div class="hoststatusdetailheader">
                <div class="hostimage">
                    <!--image-->
                    <?php show_object_icon($host, "", true); ?>
                </div>
                <div class="hosttitle">
                    <div class="hostname"><?php echo htmlentities($host); ?></div>
                </div>
            </div>
            <div class="clear"></div>
        <?php }
    } else {
        ?>
        <h1><?php echo gettext("Alert Histogram"); ?></h1>

        <?php
        if (!empty($service)) {
            ?>
            <div class="servicestatusdetailheader">
                <div class="serviceimage">
                    <!--image-->
                    <?php show_object_icon($host, $service, true); ?>
                </div>
                <div class="servicetitle">
                    <div class="servicename">
                    <?php if ($service == '*') { ?>
                        <?php echo gettext("Host Only"); ?>
                    <?php } else { ?>
                        <a href="<?php echo get_service_status_detail_link($host, $service); ?>"><?php echo htmlentities($service); ?></a>
                    <?php } ?>
                    </div>
                    <div class="hostname"><a
                            href="<?php echo get_host_status_detail_link($host); ?>"><?php echo htmlentities($host); ?></a>
                    </div>
                </div>
            </div>
            <br clear="all">

        <?php
        } else if (!empty($host)) {
            $host_object = get_xml_host_objects(array("host_name" => $host));
            ?>
            <div class="hoststatusdetailheader">
                <div class="hostimage">
                    <!--image-->
                    <?php show_object_icon($host, "", true); ?>
                </div>
                <div class="hosttitle">
                    <div class="hostname"><a
                            href="<?php echo get_host_status_detail_link($host); ?>"><?php echo htmlentities($host); ?></a> <?php if((string)$host_object->host->alias != $host) echo "( ".$host_object->host->alias." )" ?>
                    </div>
                </div>
            </div>
            <br clear="all">
        <?php
        }
        ?>
        <form method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]); ?>">

        <div class="reportexportlinks">
            <?php echo get_add_myreport_html(gettext("Alert Histogram"), $_SERVER["REQUEST_URI"], array()); ?>
            <?php
            $url = "?1";
            foreach ($request as $var => $val)
                $url .= "&" . urlencode($var) . "=" . urlencode($val);
            ?>
            <a href="<?php echo $url; ?>&mode=csv" alt="<?php echo gettext("Export As CSV"); ?>"
               title="<?php echo gettext("Export As CSV"); ?>"><img src="<?php echo theme_image("csv.png"); ?>"></a>
            <a href="<?php echo $url; ?>&mode=pdf" alt="<?php echo gettext("Download As PDF"); ?>"
               title="<?php echo gettext("Download As PDF"); ?>"><img src="<?php echo theme_image("pdf.png"); ?>"></a>
        </div>

        <!--
	<div class="reportsearchbox">
<?php
        // search box
        $searchclass = "textfield";
        if (have_value($search) == true) {
            $searchstring = $search;
            $searchclass .= " newdata";
        } else
            $searchstring = $lstr['SearchBoxText'];
        ?>
	<input type="text" size="15" name="search" id="searchBox" value="<?php echo encode_form_val($searchstring); ?>" class="<?php echo $searchclass; ?>" />
	</div>
	//-->

        <?php
        $auto_start_date = get_datetime_string(strtotime('yesterday'), DT_SHORT_DATE);
        $auto_end_date = get_datetime_string(strtotime('Today'), DT_SHORT_DATE);
        ?>

        <script type="text/javascript">
            $(document).ready(function () {
                $('#startdateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });
                $('#enddateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });


                showhidedates();

                $('#reportperiodDropdown').change(function () {
                    showhidedates();
                });

                function showhidedates() {
                    if ($('#reportperiodDropdown').val() == 'custom')
                        $('#customdates').show();
                    else
                        $('#customdates').hide();
                }

            });
        </script>

        <div class="reporttimepicker">
            <div style="float: left; padding-right: 10px;">
                <?php echo gettext("Period"); ?>&nbsp;
                <select id='reportperiodDropdown' name="reportperiod">
                    <?php
                    $tp = get_report_timeperiod_options();
                    foreach ($tp as $shortname => $longname) {
                        echo "<option value='" . $shortname . "' " . is_selected($shortname, $reportperiod) . ">" . $longname . "</option>";
                    }
                    ?>
                </select>
            </div>

            <div id="customdates" style="float: left;">
                &nbsp;<?php echo gettext("From"); ?>&nbsp;
                <input class="textfield" type="text" id='startdateBox' name="startdate"
                       value="<?php echo $startdate; ?>" size="16"/>

                <div class="reportstartdatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="startdatepickercontainer"></div>
                &nbsp;<?php echo gettext("To"); ?>&nbsp;
                <input class="textfield" type="text" id='enddateBox' name="enddate" value="<?php echo $enddate; ?>"
                       size="16"/>

                <div class="reportenddatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="enddatepickercontainer"></div>
                &nbsp;
            </div>
        </div>

        <div class="reportoptionpicker" style="clear: left;">
            <?php echo gettext("Limit To"); ?>&nbsp;
            <select name="host" id="hostList" style="width: 150px;">
                <option value=""><?php echo gettext("Host"); ?>:</option>
                <?php
                $args = array('brevity' => 1, 'orderby' => 'host_name:a');
                $oxml = get_xml_host_objects($args);
                if ($oxml) {
                    foreach ($oxml->host as $hostobject) {
                        $name = strval($hostobject->host_name);
                        echo "<option value='" . $name . "' " . is_selected($host, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
            <select name="service" id="serviceList" style="width: 150px; <?php if (empty($service) && empty($host)) { echo 'display: none;'; } ?>">
                <option value="">[<?php echo gettext("All Services"); ?>]</option>
                <option value="*" <?php if ($service == '*') { echo 'selected'; } ?>>[<?php echo gettext("Host Only"); ?>]</option>
                <?php
                $args = array('brevity' => 1, 'host_name' => $host, 'orderby' => 'service_description:a');
                $oxml = get_xml_service_objects($args);
                if ($oxml) {
                    foreach ($oxml->service as $serviceobj) {
                        $name = strval($serviceobj->service_description);
                        echo "<option value='" . $name . "' " . is_selected($service, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
            <select name="hostgroup" id="hostgroupList" style="width: 150px;">
                <option value=""><?php echo gettext("Hostgroup"); ?>:</option>
                <?php
                $args = array('orderby' => 'hostgroup_name:a');
                $oxml = get_xml_hostgroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->hostgroup as $hg) {
                        $name = strval($hg->hostgroup_name);
                        echo "<option value='" . $name . "' " . is_selected($hostgroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
            <select name="servicegroup" id="servicegroupList" style="width: 150px;">
                <option value=""><?php echo gettext("Servicegroup"); ?>:</option>
                <?php
                $args = array('orderby' => 'servicegroup_name:a');
                $oxml = get_xml_servicegroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->servicegroup as $sg) {
                        $name = strval($sg->servicegroup_name);
                        echo "<option value='" . $name . "' " . is_selected($servicegroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
        </div>

        <div class="reportoptionpicker" style="clear: left; margin-top: -25px;">

            <?php echo gettext("Group By"); ?>&nbsp;
            <select id='groupbyDropdown' name="groupby">
                <option
                    value='hour_of_day' <?php echo is_selected("hour_of_day", $groupby); ?>><?php echo gettext("Hour of Day"); ?></option>
                <option
                    value='day_of_week' <?php echo is_selected("day_of_week", $groupby); ?>><?php echo gettext("Day of Week"); ?></option>
                <option
                    value='day_of_month' <?php echo is_selected("day_of_month", $groupby); ?>><?php echo gettext("Day of Month"); ?></option>
                <option
                    value='month' <?php echo is_selected("month", $groupby); ?>><?php echo gettext("Month"); ?></option>
            </select>
            &nbsp;<?php echo gettext("State Types"); ?>&nbsp;
            <select id='statetypeDropdown' name="statetype">
                <option
                    value='soft' <?php echo is_selected("soft", $statetype); ?>><?php echo gettext("Soft"); ?></option>
                <option
                    value='hard' <?php echo is_selected("hard", $statetype); ?>><?php echo gettext("Hard"); ?></option>
                <option
                    value='both' <?php echo is_selected("both", $statetype); ?>><?php echo gettext("Both"); ?></option>
            </select>

            <input type='submit' class='reporttimesubmitbutton' name='reporttimesubmitbutton'
                   value='<?php echo gettext("Update"); ?>'>

            <!-- Set a variable to let us know it's okay to run this -->
            <input type="hidden" name="manual_run" value="1">

        </div>

        <?php
        // Die right here if we don't want to auto-load the page
        if ($disable_report_auto_run == 1 && $manual_run == 0) {
            die();
        }
        ?>

        <div style="clear: left; padding-top: 10px;">
            <?php echo gettext("Report Covers"); ?>&nbsp;<?php echo gettext("From"); ?>:
            <b><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b> <?php echo gettext("To"); ?>
            : <b><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b>
        </div>

    <?php

    } // end hideoptions

    $url = "histogram.php?mode=image";
    foreach ($request as $var => $val)
        $url .= "&" . urlencode($var) . "=" . urlencode($val);
    ?>
    <br/>
    <script type="text/javascript">
        $(document).ready(function () {
            $('#hostList').searchable({maxMultiMatch: 9999});
            $('#hostgroupList').searchable({maxMultiMatch: 9999});
            $('#servicegroupList').searchable({maxMultiMatch: 9999});   

            $('#hostList').change(function () {
                $('#hostgroupList').val('');
                $('#servicegroupList').val('');

                if ($(this).val() != '') {
                    update_service_list();
                    $('#serviceList').show();
                } else {
                    $('#serviceList').val('').hide();
                }
            });

            $('#servicegroupList').change(function () {
                $('#hostList').val('');
                $('#hostgroupList').val('');
                $('#serviceList').val('').hide();
            });

            $('#hostgroupList').change(function () {
                $('#servicegroupList').val('');
                $('#hostList').val('');
                $('#serviceList').val('').hide();
            });

            function update_service_list() {

            }

            <?php if($hideoptions): ?>
            var tracking = false;
            <?php else: ?>
            var tracking = true;
            <?php endif; ?>

            $.getJSON('<?php echo $url; ?>',
                function (data) {
                    var chart;
                    var options = {
                        'plotOptions': {'series': {'enableMouseTracking': tracking, 'animation': false}},
                        'chart': {
                            'renderTo': 'alert_histogram_image',
                            'width': 600,
                            'height': 225
                        },
                        'credits': {'enabled': false},
                        'title': {'text': data.graph_title},
                        'xAxis': {
                            'categories': data.categories,
                            'title': {'text': data.x_label},
                            'labels': {
                                'rotation': -45,
                                'align': 'right'
                            }
                        },
                        'yAxis': {
                            'title': {'text': data.y_label},
                            'min': 0
                        },
                        'series': [
                            {
                                'data': data.data,
                                'name': data.name
                            }
                        ]
                    }
                    chart = new Highcharts.Chart(options);
                });
        });
    </script>
    <div id='alert_histogram_image'>
        <!--
		<img src="<?php echo $url; ?>&mode=image" border="0" />
-->
    </div>

    </form>

    <?php

    /*
        $xml=get_histogram_xml();
        print_r($xml);
        // build array
        $xdata=array();
        $ydata=array();
        process_histogram_xml($xml,$xdata,$ydata,$xtitle);
        echo "<BR>";
        echo "<BR>X=<BR>";
        print_r($xdata);
        echo "<BR>Y=<BR>";
        print_r($ydata);
    */

    // closes the HTML page
    do_page_end(true);
}


// this function gets the XML records of state history data for multiple
// output formats (CSV, PDF, HTML)
/**
 * @return SimpleXMLElement
 */
function get_histogram_xml()
{
    global $lstr;

    // makes sure user has appropriate license level
    licensed_feature_check();

    // get values passed in GET/POST request
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");
    $search = grab_request_var("search", "");
    $host = grab_request_var("host", "");
    $service = grab_request_var("service", "");
    $hostgroup = grab_request_var("hostgroup", "");
    $servicegroup = grab_request_var("servicegroup", "");
    $groupby = grab_request_var("groupby", "hour_of_day");
    $statetype = grab_request_var("statetype", "both");

    // fix search
    if ($search == $lstr['SearchBoxText'])
        $search = "";

    // special "all" stuff
    if ($hostgroup == "all")
        $hostgroup = "";
    if ($servicegroup == "all")
        $servicegroup = "";
    if ($host == "all")
        $host = "";

    // can do hostgroup OR servicegroup OR host
    if ($hostgroup != "") {
        $servicegroup = "";
        $host = "";
    } else if ($servicegroup != "") {
        $host = "";
    }

    //  limit hosts by hostgroup or host
    $host_ids = array();
    //  limit by hostgroup
    if ($hostgroup != "") {
        $host_ids = get_hostgroup_member_ids($hostgroup);
    }
    //  limit by host
    //else if($host!=""){
    ///	$host_ids[]=get_host_id($host);
    //	}
    //  limit service by servicegroup
    $service_ids = array();
    if ($servicegroup != "") {
        $service_ids = get_servicegroup_member_ids($servicegroup);
    }

    $object_ids_str = "";
    $y = 0;
    foreach ($host_ids as $hid) {
        if ($y > 0)
            $object_ids_str .= ",";
        $object_ids_str .= $hid;
        $y++;
    }
    foreach ($service_ids as $sid) {
        if ($y > 0)
            $object_ids_str .= ",";
        $object_ids_str .= $sid;
        $y++;
    }


    // determine start/end times based on period
    get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

    // get XML data from backend - the most basic example
    // this will return all records (no paging), so it can be used for CSV export
    $args = array(
        "starttime" => $starttime,
        "endtime" => $endtime,
        "groupby" => $groupby,
    );
    switch ($statetype) {
        case "soft":
            $args["state_type"] = 0;
            break;
        case "hard":
            $args["state_type"] = 1;
            break;
        default:
            break;
    }
    // object id limiters
    if ($object_ids_str != "")
        $args["object_id"] = "in:" . $object_ids_str;
    else {
        if (!empty($host)) {
            $args["host_name"] = strval($host);
            if ($service == '*') {
                $args["objecttype_id"] = OBJECTTYPE_HOST;
            }
        }
        if (!empty($service) && $service != '*') {
            $args["service_description"] = $service;
            $args["objecttype_id"] = OBJECTTYPE_SERVICE;
        }
    }
    if ($search) {
        $args["output"] = "lk:" . $search;
    }

    //echo "ARGS:<BR>";
    //print_r($args);
    $xml = get_histogram_data($args);
    //print_r($xml);
    //exit();
    return $xml;
}

/**
 * @param $xml
 * @param $xarr
 * @param $yarr
 * @param $xtitle
 */
function process_histogram_xml($xml, &$xarr, &$yarr, &$xtitle)
{
    $groupby = grab_request_var("groupby", "hour_of_day");

    // Setup titles and X-axis labels
    $xtitle = "";
    switch ($groupby) {
        case "month":
            $xtitle = gettext("Month");
            $buckets = 12;
            break;
        case "day_of_month":
            $xtitle = gettext("Day of the Month");
            $buckets = 31;
            break;
        case "day_of_week":
            $xtitle = gettext("Day of the Week");
            $buckets = 7;
            break;
        case "hour_of_day":
            $xtitle = gettext("Hour of the Day");
            $buckets = 24;
            break;
        default:
            break;
    }

    // initialize arrays
    for ($x = 0; $x < $buckets; $x++) {

        $yarr[] = 0;

        switch ($groupby) {
            case "month":
                // We need to iterate all the way through for yarr, so if its
                // day_of_week, just initialize it, then skip the rest of
                // the iteration.
                if (empty($xarr) === TRUE) {
                    $xarr = array("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
                }
                break;
            case "day_of_month":
                $xarr[] = ($x);
                break;
            case "day_of_week":
                // We need to iterate all the way through for yarr, so if its
                // day_of_week, just initialize it, then skip the rest of
                // the iteration.
                if (empty($xarr) === TRUE) {
                    $xarr = array("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun");
                }
                break;
            case "hour_of_day":
                // We need to iterate all the way through for yarr, so if its
                // day_of_week, just initialize it, then skip the rest of
                // the iteration.
                if (empty($xarr) === TRUE) {
                    $xarr = array("12am", "1am", "2am", "3am", "4am", "5am", "6am", "7am", "8am", "9am", "10am", "11am", "Noon", "1pm", "2pm", "3pm", "4pm", "5pm", "6pm", "7pm", "8pm", "9pm", "10pm", "11pm");
                }
                break;
            default:
                break;
        }
    }
    // extra one for hour of day (jpgraph bug)
    //if($groupby=="hour_of_day")
    //	$xarr[24]=24;

    // insert real data
    if ($xml) {
        foreach ($xml->histogramelement as $he) {

            $total = intval($he->total);

            $month = intval($he->month);
            $day_of_month = intval($he->day_of_month);
            $day_of_week = intval($he->day_of_week);
            $hour_of_day = intval($he->hour_of_day);

            $index = 0;
            switch ($groupby) {
                case "month":
                    $index = $month - 1;
                    break;
                case "day_of_month":
                    $index = $day_of_month - 1;
                    break;
                case "day_of_week":
                    $index = $day_of_week;
                    break;
                case "hour_of_day":
                    $index = $hour_of_day;
                    break;
                default:
                    break;
            }

            $yarr[$index] = $total;
        }
    }

    // last bucket is same as first (only for hour of day)
    //if($groupby=="hour_of_day")
//		$yarr[24]=$yarr[0];
    //$yarr[$buckets]=$yarr[0];
}

/**
 * @param $val
 *
 * @return mixed
 */
function histogram_xaxis_callback($val)
{

    $groupby = grab_request_var("groupby", "hour_of_day");
//~ print $val;
    switch ($groupby) {
        case "month":
            $month = $val % 12;
            $months = array(gettext("Jan"), gettext("Feb"), gettext("Mar"), gettext("Apr"), gettext("May"), gettext("Jun"), gettext("Jul"), gettext("Aug"), gettext("Sep"), gettext("Oct"), gettext("Nov"), gettext("Dec"));
            $out = $months[$month];
            break;
        case "day_of_month":
            $out = $val + 1;
            break;
        case "day_of_week":
            $day = $val % 7;
            $days = array(gettext("Mon"), gettext("Tue"), gettext("Wed"), gettext("Thu"), gettext("Fri"), gettext("Sat"), gettext("Sun"));
            $out = $days[$day];
            break;
        case "hour_of_day":
            $hour = $val % 24;
            $hours = array(gettext("12am"), gettext("1am"), gettext("2am"), gettext("3am"), gettext("4am"), gettext("5am"), gettext("6am"), gettext("7am"), gettext("8am"), gettext("9am"), gettext("10am"), gettext("11am"), gettext("Noon"), gettext("1pm"), gettext("2pm"), gettext("3pm"), gettext("4pm"), gettext("5pm"), gettext("6pm"), gettext("7pm"), gettext("8pm"), gettext("9pm"), gettext("10pm"), gettext("11pm"));
            $out = $hours[$hour];
            //$out=$val;
            break;
        default:
            $out = $val;
            break;
    }

    return $out;
}


// this function generates json for a histogram highchart
function get_histogram_image()
{


    $xml = get_histogram_xml();

    // build array
    $xdata = array();
    $ydata = array();
    process_histogram_xml($xml, $xdata, $ydata, $xtitle);

    // Turn the xy points into datasets for Highcharts
    $categories = $xdata;
    $data = $ydata;
    $name = gettext('Alerts');
    $gtitle = sprintf(gettext('Alerts by %s'), $xtitle);
    $ytitle = gettext('Alerts');

    $json = array('categories' => $categories,
        'data' => $data,
        'name' => $name,
        'graph_title' => $gtitle,
        'y_label' => $ytitle,
        'x_label' => $xtitle);

    header("Content-type: application/json");

    print json_encode($json);
    return;

}


// this function generates a CSV file of histogram data
function get_histogram_csv()
{
    $groupby = grab_request_var("groupby", "hour_of_day");


    $xml = get_histogram_xml();
    // build array
    $xdata = array();
    $ydata = array();
    process_histogram_xml($xml, $xdata, $ydata, $xtitle);

    // output header for csv
    header("Content-type: application/octet-stream");
    header("Content-Disposition: attachment; filename=\"histogram.csv\"");

    // column definitions
    echo $groupby . ",total_alerts\n";

    //print_r($xml);
    //exit();

    $x = 0;
    foreach ($xdata as $xd) {
        echo $xd . "," . $ydata[$x] . "\n";
        $x++;
    }
    /*
    if($xml){
        foreach($xml->histogramelement as $he){

            // what type of log entry is this?  we change the image used for each line based on what type it is...
            //$object_type=intval($he->objecttype_id);
            //$host_name=strval($he->host_name);
            //$service_description=strval($he->service_description);
            $total=intval($he->total);
            $month=intval($he->month);
            $dom=intval($he->day_of_month);
            $dow=intval($he->day_of_week);
            $hod=intval($he->hour_of_day);

            $index=0;
            switch($groupby){
                case "month":
                    $index=$month;
                    break;
                case "day_of_month":
                    $index=$dom;
                    break;
                case "day_of_week":
                    $index=$dow;
                    break;
                case "hour_of_day":
                    $index=$hod;
                    break;
                default:
                    break;
                }


            echo $total.",".$index."\n";
            }
        }
    */
}


// This function generates a PDF of histogram data.
// Requires the "fpdp.php" library and the "fonts" directory for font metrics.
function get_histogram_pdf()
{
    global $cfg;

    //get user name
    $username = $_SESSION["username"];
    // get backend ticket
    $backend_ticket = get_user_attr(0, "backend_ticket");

    // Assemble actual URL that will be gotten
    $prefix = ($cfg['use_https'] == true) ? 'https' : 'http';
    $uri = str_replace("mode=pdf", "hideoptions=1", $_SERVER["REQUEST_URI"]);
    $fullurl = "{$prefix}://127.0.0.1{$uri}";
    //echo "FULLURL: $fullurl\n";

    $urlparts = parse_url($fullurl);

    $newurl = "";
    $newurl .= get_internal_url() . "/reports/histogram.php";
    $newurl .= "?";
    if (isset($urlparts['query']))
        $newurl .= $urlparts['query'];

    $newurl .= "&username=" . $username;
    $newurl .= "&ticket=" . $backend_ticket;

    //echo "NEWURL: $newurl\n";

    // Add language to url
    $language = $_SESSION['language'];
    $newurl .= "&locale=" . $language;

    // Temporary fix for num records
    //$newurl.="&records=100000"; // Max: 100,000 state changes

    // Do page rendering

    $aurl = $newurl;

    $afile = "page.pdf";
    $fname = get_tmp_dir() . "/scheduledreport-" . $username . "-" . time() . "-" . $afile;

    $cmdft = '--footer-spacing 3 --margin-bottom 15mm --footer-font-size 9 --footer-right "Page [page] of [toPage]" --footer-left "' . get_datetime_string(time(), DT_SHORT_DATE_TIME, DF_AUTO, "null") . '"';

    // --orientation landscape
    $cmd = "/usr/bin/wkhtmltopdf -O landscape --disable-smart-shrinking --no-outline {$cmdft} '{$aurl}' '{$fname}' 2>&1";
    @exec($cmd);

    if (!file_exists($fname)) {
        echo "\n\n************\nERROR: Failed to render URL '" . $aurl . "' as '" . $fname . "'\n************\n\n";
    } else {
        // We'll be outputting a PDF
        header('Content-type: application/pdf');
        header('Content-Disposition: inline; filename="histogram.pdf"');

        // The PDF source is in original.pdf
        readfile($fname);
        unlink($fname);
    }
}

?>